package client;

import java.io.*;

public class ClientCSVWriter {	
  private char ENDFIELD = '\n';
  private char ENDBLOCK = ';';
  private Writer writer;
  private boolean written;

  public ClientCSVWriter(Writer wtr) {
  	this.writer = wtr;
  }

  public Writer getWriter() {
    return this.writer;
  }

  public void setFieldDelimiter(char ch) {
    ENDFIELD = ch;
  }

  public void setBlockDelimiter(char ch) {
    ENDBLOCK = ch;
  }

  public void writeField(String field) throws IOException {
    if (written) {
      writer.write(ENDFIELD);
      written = false;
    }
    int index = field.indexOf(ENDFIELD);
    if (index != -1) {
      field = "\""+replace(field,"\"","\"\"")+"\"";
    }
    writer.write(field);
    written = true;
  }

  public void endBlock() throws IOException {
    writer.write(ENDBLOCK);
    written = false;
	}
    
  public void endLine() throws IOException {
    writer.write(ENDFIELD);
  }

  public void writeLine(String[] strs) throws IOException {
    int sz = strs.length;
    for(int i=0;i<sz;i++)
      writeField(strs[i]);
    endBlock();
  }

  public void close() throws IOException {
    this.writer.close();
  }

  private String replace(String text, String repl, String with) {   
   	int max = -1;
    if (text == null || repl == null || with == null || repl.length() == 0 || max == 0) {
      return text;
    }
        
    StringBuffer buf = new StringBuffer(text.length());
    int start = 0, end = 0;
    while ((end = text.indexOf(repl, start)) != -1) {
      buf.append(text.substring(start, end)).append(with);
      start = end + repl.length();
            
      if (--max == 0)
        break;
    }
    buf.append(text.substring(start));
    return buf.toString();
  }
}


