package client;

import java.io.*;

/**
 * The ClientCluster class represents a cluster.
 */
public class ClientCluster implements Serializable {
	private int clusterCode;
	private int unigeneCode;
	private int nbESTTotal;
	private int nbGO;
	private String desc;
		
	/**
	 * Construct a new ClientCluster object.
	 * @param clusterCode the TIGR cluter code
	 * @param unigeneCode the NCBI Unigene gene code
	 * @param nbESTTotal the total number of EST composing this cluster
	 * @param nbGO the nb of GO-Functions for this cluster
	 */
	public ClientCluster(int clusterCode, int unigeneCode, 
																									int nbESTTotal, int nbGO) {		
		this.clusterCode = clusterCode;
		this.unigeneCode = unigeneCode;
		this.nbESTTotal = nbESTTotal;
		this.nbGO = nbGO;
		this.desc = "";
	}
	
	/**
	 * Construct a new ClientCluster object.
	 * @param clusterCode the TIGR cluter code
	 * @param unigeneCode the NCBI Unigene gene code
	 * @param nbESTTotal the total number of EST composing this cluster
	 * @param nbGO the nb of GO-Functions for this cluster
	 * @param desc the description associated with this cluster
	 */
	public ClientCluster(int clusterCode, int unigeneCode, int nbESTTotal, 
																										int nbGO, String desc) {		
		this.clusterCode = clusterCode;
		this.unigeneCode = unigeneCode;
		this.nbESTTotal = nbESTTotal;
		this.nbGO = nbGO;
		this.desc = desc;
	}

	/**
	 * Return the TIGR cluster code.
	 * @return the TIGR cluster code
	 */
	public int getClusterCode() {
		return this.clusterCode;
	}
	
	/**
	 * Return the NCBI Unigene gene code.
	 * @return the NCBI Unigene gene code
	 */
	public int getUnigeneCode() {
		return this.unigeneCode;
	}

	/**
	 * Return the total number of EST composing this cluster.
	 * @return the total number of EST composing this cluster
	 */	
	public int getNbESTTotal() {
		return this.nbESTTotal;
	}
	
	/**
	 * Return the number of Go-Functions for this cluster.
	 * @return the number of Go-Functions for this cluster.
	 */	
	public int getNbGoFunctions() {
		return this.nbGO;
	}
			
	/**
	 * Return the description associated with this cluster.
	 * @return the description associated with this cluster
	 */
	public String getDescription() {
		return this.desc;
	}
	
	/**
	 * Set the description associated with this cluster.
	 * @param desc the description associated with this cluster to set
	 */	
	public void setDescription(String desc) {
		this.desc = desc;
	}

	/**
	 * Get in a string the representation of the content of the cluster.
	 * @return the representation of the content of the cluster
	 */
	public String toString() {
		String ret = "Cluster -> ";
		ret += "Cluster Code : "+clusterCode+" ";
		ret += ", nbESTTotal : "+nbESTTotal+" ";
		ret += ", nbGO : "+nbGO+" ";
		ret += ", Description : "+desc+     " ";
		return ret;
	}
}