package client;

import java.net.URL;
import java.sql.*;
import java.util.HashMap;
import java.util.StringTokenizer;

/**
 * The ClientConnectionTest is used to test connection parameters.
 */
public class ClientConnectionTest {
	private ClientUniversalDB db;		
	private Connection con;
	private ClientParameters cp;
	private boolean ok = false;

  /**
   * Default constructor for ClientConnectionTest. 
   * Create a new ClientConnectionTest object which try to connect to the database.
   * @param login the server login
   * @param password the server password
   * @param ipServer the ip address of the server
   * @param port the database access port
   * @param cp the current parameters for the program
   */
  public ClientConnectionTest(String login, String password, String ipServer, 
  														String port, ClientParameters cp) {	
		try { 
			this.cp = cp;
			this.db = new ClientUniversalDB(
				login, password, "mysql", "bio_db", ipServer, port);
			this.con = db.connect();  	
			ok = true;
    }
    catch (SQLException e) { 
    	PrintConnectionError();
    	PrintSQLError(e); 
    } 
    catch (Exception e) {
    	PrintConnectionError(); 
    }    	
	}
	
	/**
	 * Return the Connection established 
	 *@return Connection established or return null if problem
	 */
	public Connection getConnection() {
		if (ok) return this.con;
		else return null;
	}

	/**
	 * Print a connection error text when connection is impossible.
	 */
	public void PrintConnectionError() {
		if (cp.PRINT_DEBUG_INFO) {
			System.out.println("ERROR : Can't connect to specified database");
	    System.out.println("  Possible Source of this problem :");
	    System.out.println("   - Please check your database connection parameters");
	    System.out.println("   - Please check your network connection parameters");
	    System.out.println("   - MySQL Server may be down");
	    System.out.println();	
	  }
  }
	
	/**
	 * Print a catched SQL Exception.
	 * @param ex the SQL exception to print
	 */
	public void PrintSQLError(SQLException ex) {
		if (cp.PRINT_DEBUG_INFO) {
		  System.out.println("** ERROR SQLException :");
		  while (ex != null) { 
		   	System.out.println("*  State  : " + ex.getSQLState());
		    System.out.println("*  Message: " + ex.getMessage());
		    System.out.println("*  Error  : " + ex.getErrorCode());
		    ex = ex.getNextException();
		    System.out.println();
		  }
		}
	}	
}

