package client;

import java.io.*;

/**
* The class ClientEST represents an 'expressed sequence tag' (EST)
*/
public class ClientEST implements Serializable {
	private String ESTCode;
	private int lib;
	private int tissue;
	private int devStage;
	
	/**
	 * Construct a new ClientEST object.
	 * @param ESTCode the EST code.
	 * @param lib the library code
	 * @param tissue the tissue code
	 * @param devStage the development stage code
	 */
	public ClientEST(String ESTCode, int lib, int tissue, int devStage) {		
		this.ESTCode = ESTCode;
		this.lib = lib;	
		this.tissue = tissue;
		this.devStage = devStage;
	}

	/**
	 * Get the EST code of this EST.
	 * @return an integer giving the EST code.
	 */
	public String getESTCode() {
	  return this.ESTCode;
	}
	
	/**
	 * Get the library code of this EST.
	 * @return an integer giving the library code.
	 */
	public int getLibrary() {
	  return this.lib;
	}
	
	/**
	 * Get the tissue code of this EST.
	 * @return an integer giving the tissue code.
	 */
	public int getTissue() {
	  return this.tissue;
	}
	
	/**
	 * Get the development stage code of this EST.
	 * @return an integer giving the development stage code.
	 */
	public int getDevStage() {
	  return this.devStage;
	}
	
	/**
	 * Get in a string the representation of the content of this EST.
	 * @return the representation of the content of the EST
	 */
	public String toString() {
		String ret = " EST -> ";
		ret += " EST Code : "+ESTCode+   " ";
		ret += ", Library : "+lib+       " ";
		ret += ", Tissue : "+tissue+     " ";
		ret += ", DevStage : "+devStage+ " ";
		return ret;
	}
}