package client;

import java.io.*;
import java.util.ArrayList;

/**
 * A ClientESTSet object is a set of EST which are associated
 *	 with a cluster.
 */	  
public class ClientESTSet implements Serializable {
	private int clusterCode;
	private ArrayList list;
		
	/**
	 * Construct a new ClientESTSet object.
	 * @param clusterCode the associated cluster code
	 */	
	public ClientESTSet(int clusterCode) {
		this.clusterCode = clusterCode;
		list = new ArrayList();
	}
	
	/**
	 * Construct a new ClientESTSet object.
	 * @param clusterCode the associated cluster code
	 * @param list the list of EST associated with the cluster code
	 */			
	public ClientESTSet(int clusterCode, ArrayList list) {
		this.clusterCode = clusterCode;
		this.list = list;
	}
	
	/**
	 * Get the cluster code associated with the EST set.
	 * @return the cluster code
	 */
	public int getClusterCode() {
		return this.clusterCode;
	}
	
	/**
	 * Set a new cluster code associated with the EST set.
	 * @param clusterCode the code of the associated cluster to set
	 */		
	public void setClusterCode(int clusterCode) {
		this.clusterCode = clusterCode;
	}
	
	/**
	 * Get the number of EST present in the EST set.
	 * @return the number of EST in this EST set
	 */	
	public int getNbEST(){
		return list.size();
	}
	
	/**
	 * Get the EST list.
	 * @return the ArrayList containing all the EST.
	 */		
	public ArrayList getESTList() {
		return this.list;
	}
	
	/**
	 * Set a new EST list.
	 * @param list the ArrayList containing all the EST to set.
	 */		
	public void setESTList(ArrayList list){
		this.list = list;
	}
	
	/**
	 * Add a new EST in the set.
	 * @param e the EST to add in the set.
	 */	
	public void addEST(ClientEST e) {
		list.add(e);
	}
	
	/**
	 * Delete an EST from the set at a specified position.
	 * @param n the position of the EST to delete
	 */			
	public void delEST(int n) {
		if (!(n < 0 || n >= list.size()))	
			list.remove(n);
	}
	
	/**
	 * Get the EST at speciefied position in the set.
	 * @param n the position of the EST in the set
	 * @return the EST at the specified position 
	 */
	public ClientEST getEST(int n) {
		return ((ClientEST)list.get(n));
	}
	
	/**
	 * Get in a string the representation of the content of the EST set.
	 * @return the representation of the content of the EST set
	 */	
	public String toString() {
		if(list.isEmpty()) return "ESTSet -> *Empty*";
		else {
			String ret = "ESTSet for Cluster "+clusterCode+" -> \n";
			for (int i = 0; i < list.size(); ++i)
			  ret += list.get(i).toString() + "\n";
			return ret;
		}
	}
}