package client;

import java.io.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.border.TitledBorder;

/**
 * The ClientFrameConfig object is an extended Jframe object used 
 *	to tell user the connection parameters which are needed to connect
 *	to the data base server.
 */
public class ClientFrameConfig extends JFrame {		
	private ClientParameters cparam;
	private ClientConnectionTest cct;
	private ClientSplash splash;
	
	private JPanel sp;
	private JTextField loginT;
	private JPasswordField pwT;
	private JTextField portT;
	private JTextField ipT;	
	private JCheckBox log;
	private JButton ok;
	private JButton cancel;
	
	/**
	 * Construct a new ClientFrameConfig object.
	 */
  public ClientFrameConfig() {
    super("..::  bioECS v1.0  ::  Client Authentification Window  ::.. ");
   	setIconImage(Toolkit.getDefaultToolkit().getImage("./ico/bioecs.png"));
    addWindowListener(new WindowAdapter() {
    	public void windowClosing(WindowEvent e) {
    		System.exit(-1);
    	}
    });
    
    loadConfigFile();
    buildConnectionPanel();
		getContentPane().add(sp, BorderLayout.CENTER);   
    setSize(450, 270);
    setVisible(true);
  }
  
  /**
   * Build the connection parameter panel which 
   *	contains all the displayed objects.
   */
  private void buildConnectionPanel() { 	
  	JLabel login = new JLabel("Login : ");
    loginT = new JTextField(cparam.LOGIN, 16);   
    loginT.setFont(new Font("MONOSPACED",Font.PLAIN,11));
    
    JLabel pw = new JLabel("Password : ");
    pwT = new JPasswordField(16);  
  	pwT.setFont(new Font("MONOSPACED",Font.PLAIN,11));
  	
  	JLabel port = new JLabel("Server port : ");
    portT = new JTextField(cparam.PORT, 16);  
    portT.setFont(new Font("MONOSPACED",Font.PLAIN,11));
    
    JLabel ip = new JLabel("Server IP : ");
    ipT = new JTextField(cparam.SERVER, 16);  
    ipT.setFont(new Font("MONOSPACED",Font.PLAIN,11));
    
  	sp = new JPanel();	  
	  sp.setBorder(BorderFactory.createTitledBorder("Server Connection Parameters : ")); 
		sp.setForeground(Color.darkGray);
		
		log = new JCheckBox(
			"Print debug informations ?", cparam.PRINT_DEBUG_INFO);
		
		ok = new JButton(
			" Connect", new ImageIcon("./ico/ico-connect.png"));	
    ok.addActionListener(new ListOK());
    
    cancel = new JButton(
    	" Exit    " , new ImageIcon("./ico/ico-exit.png"));	
    cancel.addActionListener(new ListCancel());
    
		sp.setLayout(new GridBagLayout());
	  addComponent(sp, login, 0, 0, 1, 1, GridBagConstraints.EAST);
	 	addComponent(sp, loginT, 1, 0, 1, 1, GridBagConstraints.WEST);	
	 	addComponent(sp, pw, 0, 1, 1, 1, GridBagConstraints.EAST); 	
	 	addComponent(sp, pwT, 1, 1, 1, 1, GridBagConstraints.WEST);
	 	addComponent(sp, port, 0, 2, 1, 1, GridBagConstraints.EAST); 	
	 	addComponent(sp, portT, 1, 2, 1, 1, GridBagConstraints.WEST); 	
	 	addComponent(sp, ip, 0, 3, 1, 1, GridBagConstraints.EAST); 	
	 	addComponent(sp, ipT, 1, 3, 1, 1, GridBagConstraints.WEST);
	 	addComponent(sp, log, 0, 4, 2, 1, GridBagConstraints.CENTER);
	 	addComponent(sp, ok, 0, 5, 1, 1, GridBagConstraints.EAST);	  
	  addComponent(sp, cancel, 1, 5, 1, 1, GridBagConstraints.WEST); 	
  }
	
	/**
	 * Set the frame in state visible or invisible, 
	 *	and set frame location on screen according to its current size.
	 * @param b a boolean indicating the frame visibility value
	 */
	public void setVisible(boolean b){
		if (b) {
			Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		  int screenWidth  = screenSize.width;
		  int screenHeight = screenSize.height;
		  int frameWidth   = getSize().width;
		  int frameHeight  = getSize().height;
		  int posX         = (screenWidth - frameWidth) / 2;
		  int posY         = (screenHeight - frameHeight) / 2;
		  setLocation(posX, posY);
		}
		super.setVisible(b);
	}
	
	/**
	 * Load the previous saved configuration file.
	 * 	Create a new one if non exist in the current folder.
	 */
	private void loadConfigFile() {
		cparam = ClientParameters.getClientParameters("cparam.cfg");
		if (cparam == null)
			cparam = new ClientParameters();
	}

	/**
	 * Simply add a component to the container passed as argument.
	 * @param container the container to update
	 * @param component the component to add
	 * @param x the horizontal position wanted
	 * @param y the vertical position wanted
	 * @param width the number of place to take in width
	 * @param height the number of place to take in height
	 * @param position the position of the component
	 */
	private void addComponent(Container container, Component component, 
	  												int x, int y, int width, int height, int position) {
	  addWithConstrains(container, component, x, y, width, height, position);
	}
	
	/**
	 * Manage simply the layout to add component in the container. 
	 * Create a GridBagConstraints to add simply and more precisely components.
	 *	By default, the component is centered on the destination container. 
	 * @param container the container to update
	 * @param component the component to add
	 * @param x the horizontal position wanted
	 * @param y the vertical position wanted
	 * @param width the number of place to take in width
	 * @param height the number of place to take in height
	 * @param anchor the position of the component
	 */
	private void addWithConstrains(Container container, Component component, 
	    												int x, int y, int width, int height, int anchor) {
	  Insets insets = new Insets (2, 2, 2, 2);
	  GridBagConstraints gbc = new GridBagConstraints();
	    	
	  gbc.gridx = x; gbc.gridy = y;
	  gbc.gridwidth = width; gbc.gridheight = height;
	  gbc.fill = GridBagConstraints.NONE; gbc.anchor = anchor;
	  gbc.weightx = 1.0; gbc.weighty = 1.0;
	  gbc.ipadx = 0; gbc.ipady = 0;
	  gbc.insets = insets;
	
	  LayoutManager lm = container.getLayout();
	  ((GridBagLayout) lm).setConstraints(component, gbc);
	  container.add(component);
	}
	
	/**
	 * Initialize the splash screen.
	 */
	private void initSplashScreen() {
		splash = new ClientSplash(new ImageIcon("./ico/bioecs-big.png"), "");
		splash.setAll("loading...", 0);
	}
	
	/**
	 * Show the splash screen.
	 */
	private void showSplashScreen() {
		splash.setVisible(true);
		repaint();
	}
	
	/**
	 * Hide the splash screen.
	 */
	private void hideSplashScreen() {
		splash.setVisible(false);
		splash = null;
	}
	
	/**
	 * Listerner used to test connection parameters and then, 
	 *  if parameters are good, create a new ServerGUI object to run the program.
	 */
  class ListOK implements ActionListener, Runnable {
    public void actionPerformed(ActionEvent event) { 
    	new Thread(this).start();
    } 
    
    public void run() {   			    							
			String loginText = loginT.getText();
   		String pwText = String.valueOf(pwT.getPassword());
   		String portText = portT.getText();
   		String ipText = ipT.getText();
   		
   		if (log.isSelected()) cparam.PRINT_DEBUG_INFO = true;
   		else cparam.PRINT_DEBUG_INFO = false;
			
			cct = new ClientConnectionTest(
 				loginText, pwText, ipText, portText, cparam);		
 			if (cct.getConnection() != null) {			
 				try {
 					cparam.LOGIN = loginText;
 					cparam.PORT = portText;
 					cparam.SERVER = ipText;
	 				ClientParameters.saveClientParameters(cparam, "cparam.cfg");
	 				dispose();  
	 				initSplashScreen();
					showSplashScreen();
					splash.setAll("Saving Parameters...", 20); 		
					Thread.sleep(100);						
			  	splash.setAll("Creating bioECS GUI...", 60); 
	 				
	 				// run the client program     			
		   		ClientGUI cgui = new ClientGUI(cct.getConnection(), cparam); 
		   		Thread.sleep(100);		
		   		splash.setAll("Running bioECS...", 100);	 		  
			  	
			  	cgui.setVisible(true);
			  	cgui.refreshPanelSize();
			  	hideSplashScreen();
		   	}
		   	catch(Exception e) { }
 			}
 			else {
 				connectionFailedFrame(ClientFrameConfig.this);
 				repaint();	
 			}  		  	   		    	
    }
  }
      
  /**
	 * Show a connection failed frame displaying error possible sources.
	 * @param frame the parent frame associated.
	 */
  private void connectionFailedFrame(JFrame frame) {
    JOptionPane.showMessageDialog(frame, cparam.CONNECTION_ERROR, 
    	"Connection failed !", JOptionPane.WARNING_MESSAGE, 
    	new ImageIcon("./ico/bioecs.png"));     
  }
	
	/**
	 * Listerner used to exit the program.
	 */
  class ListCancel implements ActionListener {
    public void actionPerformed(ActionEvent event) {
      System.exit(-1);
    }
  }
}