package client;

import java.io.*;
import java.util.ArrayList;
	
/**
 * A ClientGOSet object is a set of Go-Functions which are associated
 *	 with a cluster.
 */	
public class ClientGOSet implements Serializable {
	private int clusterCode;
	private ArrayList list;
	
	/**
	 * Construct a new ClientGOSet object.
	 * @param clusterCode the associated cluster code
	 */	
	public ClientGOSet(int clusterCode) {
	  this.clusterCode = clusterCode;
	  list = new ArrayList();
	}
	
	/**
	 * Construct a new ClientGOSet object.
	 * @param clusterCode the associated cluster code
	 * @param list the list of GO-Functions associated with the cluster code
	 */	
	public ClientGOSet(int clusterCode, ArrayList list) {
	  this.clusterCode = clusterCode;
	  this.list = list;
	}
	
	/**
	 * Get the cluster code associated with the GO set.
	 * @return the cluster code
	 */
	public int getClusterCode() {
	  return this.clusterCode;
	}
	
	/**
	 * Set a new cluster code associated with the GO set.
	 * @param clusterCode the code of the associated cluster to set
	 */	
	public void setClusterCode(int clusterCode) {
	  this.clusterCode = clusterCode;
	}

	/**
	 * Get the number of GO-Functions present in the GO set.
	 * @return the number of GO-Functions in this GO set
	 */
	public int getNbGOFunctions(){
	  return list.size();
	}
	
	/**
	 * Get the Go-Function list.
	 * @return the ArrayList containing all the Go-Functions.
	 */	
	public ArrayList getGOList() {
	  return this.list;
	}
	
	/**
	 * Set a new GO-Function list.
	 * @param list the ArrayList containing all the GO-Functions to set.
	 */
	public void setGOList(ArrayList list){
	  this.list = list;
	}
	
	/**
	 * Add a new Go-Function in the set.
	 * @param g the Go-Function to add in the set.
	 */
	public void addGOFunction(ClientGO g) {
	  list.add(g);
	}
	
	/**
	 * Delete a Go-Function from the set at a specified position.
	 * @param n the position of the Go-Function to delete
	 */	
	public void delGOFunction(int n) {
		if (!(n < 0 || n >= list.size()))	
	  	list.remove(n);
	}
	
	/**
	 * Get the Go-Function at speciefied position in the set.
	 * @param n the position of the Go-Function in the set
	 * @return the Go-Function at the specified position 
	 */	
	public ClientGO getGOFunction(int n) {
		return ((ClientGO)list.get(n));
	}
	
	/**
	 * Get in a string the representation of the content of the GO set.
	 * @return the representation of the content of the GO set
	 */
	public String toString() {
		if(list.isEmpty()) return "GOSet -> *Empty*";
		else {
			String ret = "GOSet for Cluster "+clusterCode+" -> \n";
			for (int i = 0; i < list.size(); ++i)
		  	ret += list.get(i).toString() + "\n";
			return ret;
		}
	}
}