package client;

import java.io.*;
import java.sql.Connection;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*; 
import javax.swing.table.*;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Timer;
import java.util.TimerTask;
import java.util.ArrayList;
import java.util.Properties;
import java.util.Enumeration;
import java.util.Calendar;
import javax.swing.filechooser.FileFilter;
import java.net.InetAddress;

/**
 * The ClientGUI's class extends a JFrame 
 * It's the graphical user interface of the bioECS client program.
 */
public class ClientGUI extends JFrame {
	private ClientFrameCloser closeFrame;		
	private ClientFrameSearch cfs;
	private ClientSearchParameterSet csp;
	private ClientThreadSearch cts;
	private ClientThreadRetrieveEST ctrest;
	private ClientThreadRetrieveGO ctrgo;
	private ClientRequestManager crm;
	private ClientParameters cp;
	
	private JToolBar toolBar;
	private JTabbedPane tabbedPane;

	private JCheckBox showESTInfo;
	private JCheckBox showGOInfo;	
	
	private JPanel panelCenter;	
	private ClientTableCluster tableCluster;
	private ClientTableEST tableEST;
	private ClientTableGO tableGO;
	
	private JPanel panelSouth;
	private ClientStatusBar statusBar;

	private HashMap HTissueWage = new HashMap();  
	private HashMap HDevStageWage = new HashMap();
	
	private JFileChooser chooser;
	private String clientIP;
	private boolean stoppedSearch = false;
	private String dir = "";
	private Connection con;

	/**
   * Default constructor for ClientGUI, Create a new ClientGUI object.
   * @param con the established database connection
   * @param cp the current parameters for the program
   */
	public ClientGUI(Connection con, ClientParameters cp) {
	  setTitle("..::  bioECS  ::  Client v1.0  ::..");
	  setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
	  setSize(getPreferredSize());	
		setIconImage(Toolkit.getDefaultToolkit().getImage("./ico/bioecs.png"));
		
		createMenuBar();
		createToolBar();
		createCenterPanel(null);
	  createSouthPanel();	
	  
	  this.cp = cp;
	  this.con = con;
	  connectToServer(con, cp);
	  add_Client_IP(clientIP);

	  loadClientFrameCloser();
	  loadAutoResizer(); 
	  createFileChooser();	  	  
	  createParametersPanel();
	  createTabbedPane();	  

	  Container c = this.getContentPane();
	  c.setLayout(new BorderLayout());
	  c.add(toolBar, BorderLayout.NORTH);	
	  c.add(tabbedPane, BorderLayout.CENTER);	
	  c.add(panelSouth, BorderLayout.SOUTH);	
	  refreshPanelSize();	 	
	}

	/**
	 * Return the frame preferred size according to the screen resolution.
	 * @return a Dimension object indicating preferred dimension
	 */
	public Dimension getPreferredSize() {
	  Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	  int screenWidth = screenSize.width;
	  int screenHeight = screenSize.height;
	  return new Dimension((screenWidth / 4) * 3, (screenHeight / 4) * 3);
	}
	
	/**
	 * Return the intern tables preferred size according to
	 *	the number of tables to display and the screen resolution.
	 * @param nbTables is the number of tables to display
	 * @return a Dimension object indicating preferred dimension
	 */
	private Dimension getPreferredTableSize(int nbTables) {
	  int screenWidth = this.getSize().width;
	  int screenHeight = this.getSize().height;	  
	  int minY = statusBar.getSize().height + 280;
	  Rectangle rectTabFirst = 
	  	tabbedPane.getUI().getTabBounds(tabbedPane, 0);
	  Rectangle rectTabLast = 
	  	tabbedPane.getUI().getTabBounds(tabbedPane, tabbedPane.getTabCount()-1);
	  
	  double headerY;
	  if (rectTabLast.getY() >= rectTabFirst.getY())
	  	headerY = rectTabLast.getY();
	  else headerY = rectTabFirst.getY();
	  String parsedDouble = Double.toString(headerY);
	  parsedDouble = parsedDouble.substring(0, parsedDouble.lastIndexOf("."));
	  int tabOffset = Integer.parseInt(parsedDouble) + 24;

	  if (nbTables == 1)
	  	return new Dimension(
	  		screenWidth - 50, (screenHeight - minY - tabOffset + 100) / nbTables);
	  else if (nbTables == 2 || nbTables == 3)
	  	return new Dimension(
	  		screenWidth - 50, (screenHeight - minY - tabOffset) / nbTables);
	  else return null;
	}
	
	/**
	 * Return the number of tables to display.
	 * @return the number of tables to display
	 */
	private int getNbTablesToDisplay() {
		int nb = 1;
		if (showESTInfo.isSelected()) ++nb;
		if (showGOInfo.isSelected())  ++nb;
		return nb;
	}
	
	/**
	 * Set the frame in state visible or invisible, 
	 *	and set frame location on screen
	 * @param b a boolean indicating the frame visibility value
	 */
	public void setVisible(boolean b){
		if (b) {
			Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		  int screenWidth  = screenSize.width;
		  int screenHeight = screenSize.height;
		  int frameWidth   = getSize().width;
		  int frameHeight  = getSize().height;
		  int posX         = (screenWidth - frameWidth) / 2;
		  int posY         = (screenHeight - frameHeight) / 2;
		  setLocation(posX, posY);
		}
		super.setVisible(b);
	}
	
	/**
   * Connect to the database server.
   * @param login the server login
   * @param pw the server password
   * @param ip the address of the server
   * @param port the database access port
   * @param cp the current client parameters for the program
   */
	private void connectToServer(Connection con, ClientParameters cp) {
		crm = new ClientRequestManager(con, cp);
		loadUpdatedAvailableInfo();
	}
	
	/**
	 * Retrieve informations about installed species in the database.
	 */
	private void loadUpdatedAvailableInfo() {
		cp.INSTALLED_SPECIES = crm.get_INSTALLED_SPECIES();
	}
	
	/**
	 * Add the client IP in the client_ip table of the data base.
	 */
	private void add_Client_IP(String ip) {
		crm.add_Client_IP(ip);
	}
	
	/**
	 * Load the frame closer designing to 'intelligently' close the application.
	 */
	private void loadClientFrameCloser() {
		closeFrame = new ClientFrameCloser();
	  addWindowListener(closeFrame);	
	}
	
	/**
	 * Load the frame's auto resizer module.
	 */
	private void loadAutoResizer() {
		this.addComponentListener(new ComponentAdapter() {
    	public void componentResized(ComponentEvent evt) { 
      	int minX = statusBar.getSize().width + 20;
      	int minY = 500;
      	int frameWidth = getSize().width;
      	int frameHeight = getSize().height;
      	if (frameWidth < minX) frameWidth = minX;
      	if (frameHeight < minY) frameHeight = minY;
      	setSize(frameWidth,frameHeight);
      	refreshPanelSize();
    	}
  	});
	}
	
	/**
	 * Refresh the intern panels size to display.
	 */
	public void refreshPanelSize() {
    Dimension size = getPreferredTableSize(getNbTablesToDisplay());
     	
    if (showESTInfo.isSelected())  		
      tableEST.setPanelSize(size);     		
    if (showGOInfo.isSelected())
      tableGO.setPanelSize(size);     	
    tableCluster.setPanelSize(size);     			     		
    repaint();
    validate();		
	}
	
	/**
	 * Create the central panel of the graphical user interface with 
	 *	initially a ClientClusterSet object to display.
	 * @param cs the ClientClusterSet to display
	 */
	private void createCenterPanel(ClientClusterSet cs) {	  
	  createClusterPanel(cs);
	  createESTPanel();
	  createGOPanel();
	  
	  panelCenter = new JPanel();
	  panelCenter.add(tableCluster, BorderLayout.NORTH); 	  
	  panelCenter.add(tableEST, BorderLayout.CENTER);
	  panelCenter.add(tableGO, BorderLayout.SOUTH);	  
	}
	
	/**
	 * Create the south panel of the graphical user interface.
	 */
	private void createSouthPanel() {	  
	  createStatusBar();
	  panelSouth = new JPanel();	  
	  panelSouth.add(statusBar, BorderLayout.SOUTH); 	  
	}
	
	/**
	 * Create the cluster panel which list all the clusters found by a search.
	 * @param cs the ClientClusterSet to display
	 */
	private void createClusterPanel(ClientClusterSet cs) {
		tableCluster = new ClientTableCluster(
			"Clusters Found", new ClientClusterSet());	
		tableCluster.addHeaderListener(new ListenerSortCluster());  
		tableCluster.addListener(new ListenerClickCluster()); 	  		
	}
	
	/**
	 * Create the EST panel which list all the EST 
	 *	composing a cluster found by a search.
	 */
	private void createESTPanel() {
		tableEST = new ClientTableEST(
			"EST details about selected cluster", new ClientESTSet(0));	
		tableEST.addHeaderListener(new ListenerSortEST());  		 
	}
	
	/**
	 * Create the GO-Functions panel which list all the GO-Functions
	 *	composing a cluster found by a search.
	 */
	private void createGOPanel() {
		tableGO = new ClientTableGO(
			"GO-Functions details about selected cluster", new ClientGOSet(0));	
		tableGO.addHeaderListener(new ListenerSortGO());  			 	  		 
	}
	
	/**
	 * Create the parameters panel of the graphical user interface.
	 */
	private void createParametersPanel() {
		if (cp.INSTALLED_SPECIES != null) {
			Object[] toShow = new Object[cp.INSTALLED_SPECIES.length];   	
	    	for (int i = 0; i < cp.INSTALLED_SPECIES.length; ++i)
	    		toShow[i] = cp.INSTALLED_SPECIES[i][6].replace('_', ' ');
	    cfs = new ClientFrameSearch(toShow);	
  	}
  	else {
  		statusBar.setTemporaryInfo(
  			" ERROR : No specie installed this moment !", 3000);
  	}
	}
	
	/**
	 * Create the tabbedPane object.
	 */
	private void createTabbedPane() {
		tabbedPane = new JTabbedPane();	  
	  tabbedPane.addTab(
	  	"Search Results", new ImageIcon("./ico/ico-result.png"), 
	  	panelCenter, "Main Search Result Panel");
		tabbedPane.setMnemonicAt(0, KeyEvent.VK_F1);
		
		if (cp.INSTALLED_SPECIES != null) {
	    tabbedPane.addTab(
				" Search Parameters ", new ImageIcon("./ico/ico-param.png"), 
				cfs, " Search Parameters Panel");
			tabbedPane.setMnemonicAt(1, KeyEvent.VK_F2);	    
		  tabbedPane.setSelectedIndex(1); 
		}    
	}
	
	
	/**
	 * Create the status box.
	 */	
	private void createStatusBar() {
		statusBar = new ClientStatusBar();
		clientIP = statusBar.getIP().replace('.', '_');
		statusBar.setTemporaryInfo(" Welcome in bioECS !", 3000);
	}
	
	/**
   * Create the Menu bar of the program.
   */
  private void createMenuBar() {
    JMenuBar menuBar = new JMenuBar();
	
	  JMenu file = new JMenu("  File  ");
	  file.setMnemonic('F');	  
	  JMenuItem load = new JMenuItem(
	  	"Load Cluster Search", new ImageIcon("./ico/ico-load.png"));
	  load.setAccelerator(KeyStroke.getKeyStroke('L', Event.CTRL_MASK));
	  load.addActionListener(new ListenerButtonLoad());	
	  file.add(load);
	  JMenuItem save = new JMenuItem(
	  	"Save Cluster Search", new ImageIcon("./ico/ico-save.png"));
	  save.setAccelerator(KeyStroke.getKeyStroke('S', Event.CTRL_MASK));
	  save.addActionListener(new ListenerButtonSave());	  
	  file.add(save);
	  
	  file.addSeparator(); 
	  
	  JMenuItem searchabort = new JMenuItem(
	  	"Abort Current Search", new ImageIcon("./ico/ico-abort.png"));
	  searchabort.setAccelerator(KeyStroke.getKeyStroke('A', Event.CTRL_MASK));
	  searchabort.addActionListener(new ListenerButtonSearchAbort());	
		file.add(searchabort);
		
	  file.addSeparator(); 
	  	     
	  JMenuItem quit = new JMenuItem(
	  	"Quit", new ImageIcon("./ico/ico-exit.png"));
	  quit.setAccelerator(KeyStroke.getKeyStroke('Q', Event.CTRL_MASK));
	  quit.addActionListener(new ListenerButtonExit());		
	  file.add(quit);
	  
	  JMenu clu_gene = new JMenu("  Cluster/Gene Tools  ");
	  clu_gene.setMnemonic('O');	  
		JMenuItem exportsearchtxt = new JMenuItem(
			"Export cluster in *.txt", new ImageIcon("./ico/ico-export-txt.png"));
	  exportsearchtxt.setAccelerator(KeyStroke.getKeyStroke('T', Event.CTRL_MASK));
		exportsearchtxt.addActionListener(new ListenerExportTXT());
		clu_gene.add(exportsearchtxt);  
		JMenuItem exportsearchcsv = new JMenuItem(
			"Export cluster in *.csv", new ImageIcon("./ico/ico-export-csv.png"));
	  exportsearchcsv.setAccelerator(KeyStroke.getKeyStroke('C', Event.CTRL_MASK));
		exportsearchcsv.addActionListener(new ListenerExportCSV());
		clu_gene.add(exportsearchcsv); 
		 
		clu_gene.addSeparator();  
		
	  JMenuItem externTIGR = new JMenuItem(
			"Cluster Info in TIGR TGI", new ImageIcon("./ico/ico-tigr-clu.png"));
	  externTIGR.setAccelerator(KeyStroke.getKeyStroke('U', Event.CTRL_MASK));
		externTIGR.addActionListener(new ListenerClusterTIGRInfo());
	  clu_gene.add(externTIGR);  
	  JMenuItem externNCBI = new JMenuItem(
			"Cluster Info in NCBI UniGene", new ImageIcon("./ico/ico-ncbi-gene.png"));
	  externNCBI.setAccelerator(KeyStroke.getKeyStroke('G', Event.CTRL_MASK));
		externNCBI.addActionListener(new ListenerUnigeneNCBIInfo());
	  clu_gene.add(externNCBI);  
	  
	  JMenu est = new JMenu("  EST Tools  ");
	  est.setMnemonic('E');
		JMenuItem externEST = new JMenuItem(
			"EST Info in TIGR TGI", new ImageIcon("./ico/ico-tigr-est.png"));
	  externEST.setAccelerator(KeyStroke.getKeyStroke('I', Event.CTRL_MASK));
		externEST.addActionListener(new ListenerESTTIGRInfo());
	  est.add(externEST);    
		
		JMenu go = new JMenu("  GO Tools  ");
		go.setMnemonic('Z');
		JMenuItem externAmiGO = new JMenuItem(
			"Open GO-Function in AmiGO", new ImageIcon("./ico/ico-amigo.png"));
	  externAmiGO.setAccelerator(KeyStroke.getKeyStroke('G', Event.CTRL_MASK));
		externAmiGO.addActionListener(new ListenerAmiGOInfo());
	  go.add(externAmiGO);
		JMenuItem externExpasy = new JMenuItem(
			"Open Protein in ExPASy", new ImageIcon("./ico/ico-expasy.png"));
	  externExpasy.setAccelerator(KeyStroke.getKeyStroke('Y', Event.CTRL_MASK));
		externExpasy.addActionListener(new ListenerExpasyInfo());
	  go.add(externExpasy);
	  
	  JMenu more = new JMenu("  ?  ");	 
	  more.setMnemonic('H');	   
	  JMenuItem about = new JMenuItem(
	  	"About", new ImageIcon("./ico/ico-about.png"));
	  about.setAccelerator(KeyStroke.getKeyStroke('B', Event.CTRL_MASK));
	  about.addActionListener(new ListenerButtonAbout());	
	  more.add(about);  
	  
	  menuBar.add(file);
	  menuBar.add(clu_gene);
	  menuBar.add(est);
	  menuBar.add(go);
	  menuBar.add(more);	  	    
		setJMenuBar(menuBar);
  }
  
  private void createToolBar() {
  	toolBar = new JToolBar();
		toolBar.setFloatable(false);
		toolBar.setRollover(true);
		
		JButton load = new JButton(new ImageIcon("./ico/ico-load.png"));
    load.setMargin(new Insets(0, 0, 0, 0));
    load.setToolTipText("Load Cluster Search");
    load.addActionListener(new ListenerButtonLoad());	
    toolBar.add(load);
    
    JButton save = new JButton(new ImageIcon("./ico/ico-save.png"));
    save.setMargin(new Insets(0, 0, 0, 0));
    save.setToolTipText("Save Cluster Search");
    save.addActionListener(new ListenerButtonSave());	
    toolBar.add(save);
    
    toolBar.addSeparator();
    
    JButton searchabort = new JButton(new ImageIcon("./ico/ico-abort.png"));
	  searchabort.setMargin(new Insets(0, 0, 0, 0));
	  searchabort.setToolTipText("Abort Current Search");
	  searchabort.addActionListener(new ListenerButtonSearchAbort());	
		toolBar.add(searchabort);
    
    toolBar.addSeparator();

		JButton exporttxt = new JButton(new ImageIcon("./ico/ico-export-txt.png"));
		exporttxt.setMargin(new Insets(0, 0, 0, 0));
		exporttxt.setToolTipText("Export selected cluster in *.txt file");
		exporttxt.addActionListener(new ListenerExportTXT());
		toolBar.add(exporttxt);
		
		JButton exportcsv = new JButton(new ImageIcon("./ico/ico-export-csv.png"));
		exportcsv.setMargin(new Insets(0, 0, 0, 0));
		exportcsv.setToolTipText("Export selected cluster in *.csv file");
		exportcsv.addActionListener(new ListenerExportCSV());
		toolBar.add(exportcsv);
		
		JButton externTIGR = new JButton(new ImageIcon("./ico/ico-tigr-clu.png"));
		externTIGR.setMargin(new Insets(0, 0, 0, 0));
		externTIGR.setToolTipText(
			"View additionnal info about the selected cluster on TIGR TGI Web Site");
		externTIGR.addActionListener(new ListenerClusterTIGRInfo());
		toolBar.add(externTIGR);
		
		JButton externNCBI = new JButton(new ImageIcon("./ico/ico-ncbi-gene.png"));
		externNCBI.setMargin(new Insets(0, 0, 0, 0));
		externNCBI.setToolTipText(
			"View additionnal info about the selected cluster on NCBI UniGene Web Site");
		externNCBI.addActionListener(new ListenerUnigeneNCBIInfo());
		toolBar.add(externNCBI);
		
		toolBar.addSeparator();
		
		JButton externEST = new JButton(new ImageIcon("./ico/ico-tigr-est.png"));
		externEST.setMargin(new Insets(0, 0, 0, 0));
		externEST.setToolTipText(
			"View additionnal info about the selected EST on TIGR TGI Web Site");
		externEST.addActionListener(new ListenerESTTIGRInfo());
		toolBar.add(externEST);
		
		toolBar.addSeparator();
		
		JButton externAmiGO = new JButton(new ImageIcon("./ico/ico-amigo.png"));
		externAmiGO.setMargin(new Insets(0, 0, 0, 0));
		externAmiGO.setToolTipText(
			"View additionnal info about the selected GO-Function on AmiGO Web Site");
		externAmiGO.addActionListener(new ListenerAmiGOInfo());
		toolBar.add(externAmiGO);
		
		JButton externExpasy = new JButton(new ImageIcon("./ico/ico-expasy.png"));
		externExpasy.setMargin(new Insets(0, 0, 0, 0));
		externExpasy.setToolTipText(
			"View additionnal info about the selected GO-Function on ExPASy Web Site");
		externExpasy.addActionListener(new ListenerExpasyInfo());
		toolBar.add(externExpasy);
		
		toolBar.addSeparator(); 
		
		showESTInfo = new JCheckBox("Show EST details", true);
		showESTInfo.setToolTipText("Show/Hide the EST table");
	  showESTInfo.addActionListener(new ListenerShowESTInfo());	
	  toolBar.add(showESTInfo);
	    
	  showGOInfo = new JCheckBox("Show GO-Functions details", true);
	  showESTInfo.setToolTipText("Show/Hide the GO-Functions table");
	 	showGOInfo.addActionListener(new ListenerShowGOInfo());
		toolBar.add(showGOInfo);
		 	
  }
  
	/**
	 * Create the File Chooser object and associate a filter to it .
	 */ 
	private void createFileChooser() {
	  chooser = new JFileChooser();
	  chooser.setCurrentDirectory(new File("."));
	  final BioFilter filter = new BioFilter();
	  chooser.setFileFilter(filter);
	}

	/** 
   * Save the ClientClusterSet result in a file.
   * @param cs the ClientClusterSet to save
   */
  public void save(ClientClusterSet cs) { 		
		int val = chooser.showSaveDialog(ClientGUI.this);
		if (val == JFileChooser.APPROVE_OPTION) {
			dir = chooser.getSelectedFile().getPath();
			File file = chooser.getSelectedFile();
			if (chooser.accept(file))
			  ClientClusterSet.saveClusterSet(cs, dir);
			else {
				dir += ".clu";
				ClientClusterSet.saveClusterSet(cs, dir);
			}
			statusBar.setTemporaryInfo(
			 	" Your search was saved in "+dir+"", 3000);
			repaint();
		}
 	}
 	
 	/** 
   * Load the ClientClusterSet result from a file.
   */
  public void load() {
    if (cts != null && cts.isAlive()) {
	  	if (confirmCancelSearch(ClientGUI.this)) {
	  		cts.stop();
		    int val = chooser.showOpenDialog(ClientGUI.this);
			  if (val == JFileChooser.APPROVE_OPTION) {
			   	dir = chooser.getSelectedFile().getPath();
			    File file = chooser.getSelectedFile();
			    if (chooser.accept(file))
			      openClusterSet(ClientClusterSet.getClusterSet(dir));
			  }
			}
		}
		else {
			int val = chooser.showOpenDialog(ClientGUI.this);
			if (val == JFileChooser.APPROVE_OPTION) {
			  dir = chooser.getSelectedFile().getPath();
			  File file = chooser.getSelectedFile();
			  if (chooser.accept(file))
			    openClusterSet(ClientClusterSet.getClusterSet(dir));
			}
		}
 	}
 	
 	/**
   * Display the result of a search which was previously loaded from a file.
   * @param cs ClientClusterSet
   */
	private void openClusterSet(ClientClusterSet cs) {	  
	  if (cs != null) {	  	
	  	if (cp.isInstalled(cs.getSpecie())) { 	  	
		  	tabbedPane.remove(cfs);
		  	cfs = new ClientFrameSearch(cs.getSearchParameterSet()); 
	   		tabbedPane.addTab(
					" Search Parameters ", new ImageIcon("./ico/ico-param.png"), 
					cfs, " Search Parameters Panel");
				tabbedPane.setMnemonicAt(1, KeyEvent.VK_F2);	    
	  		tabbedPane.setSelectedIndex(0);
	  		
	  		System.out.println(cp.getNCBIVersion(cs.getSpecie()));
	  		System.out.println(cs.getSearchParameterSet().getNCBIVersion());
	  		System.out.println(cp.getTIGRVersion(cs.getSpecie()));
	  		System.out.println(cs.getSearchParameterSet().getTIGRVersion());
	  		
		  	
		  	if (cp.getNCBIVersion(cs.getSpecie()) 
		  				>= cs.getSearchParameterSet().getNCBIVersion()
		  			&& cp.getTIGRVersion(cs.getSpecie()) 
		  				> cs.getSearchParameterSet().getTIGRVersion()) {
		  		BioUpdater update = new BioUpdater(cs);
		  	}
		  	else {
		  		tableCluster.setClusterSet(cs);  
		  		statusBar.setTemporaryInfo(
	  					" Previous search was loaded successfully !", 3000);
	  		}
			}
			else statusBar.setTemporaryInfo(
	  				" WARNING : This specie was uninstalled !", 3000);
	  }
	  else 
	  	statusBar.setTemporaryInfo(
	  		" ERROR : Can't load this previous search !", 3000);
	  repaint();
	}
	
	/**
	 * Show a confirmation frame which can cancel the current search.
	 * @param frame the parent frame associated.
	 * @return a boolean indicating the user choice to cancel or not.
	 */
  private boolean confirmCancelSearch(JFrame frame) {
    String s1 = "Yes";
    String s2 = "No";
    Object[] options = {s1, s2};
    int n = JOptionPane.showOptionDialog(frame,
    	"You must abort your current search if you want to do this task. \n"+
    	"Do you want to abort your current search ?",
    	"Abort current search ?",JOptionPane.YES_NO_OPTION, 
    	JOptionPane.QUESTION_MESSAGE, new ImageIcon("./ico/bioecs.png"), options, s1);
    if (n == JOptionPane.YES_OPTION) {
    	statusBar.setTemporaryInfo(
    		" Search aborted !", 3000);
    	stoppedSearch = true;
    	return true;
    }
    else return false;
  }
 
 	/**
	 * Show a 'Too much alignement' link frame.
	 * @param panel the parent panel associated.
	 */
   public void connectionTooMuchAlignFrame(JFrame panel) {
  	String msg = "Sorry, this link is not accessible...\n"+
  							"Too much alignements are expected to be generate !\n"+
  							"Please use your favourite WWW browser to view this page...";
    JOptionPane.showMessageDialog(panel, msg, 
    	"Too much alignements expected !", JOptionPane.WARNING_MESSAGE, 
    	new ImageIcon("./ico/bioecs.png"));     
  }
  

  /**
   * Print the message from an exception if the debug mode is selected.
   * @param ex the exception to print
   */
  public void PrintException(Exception ex) {
		if (cp.PRINT_DEBUG_INFO) {
	  	System.out.println("** CLIENT GUI Exception :");
	    System.out.println("*  Message: " +   ex.getMessage());
	  }
	}
		
		
	/***************************************************************************/	

	/**
	 * This listener manage the sorting job of the clusters table.
	 * This is run by a 'double click' on the header of the column to sort.
	 * The sorting method implemented is the 'fusion sort' which sort 
	 *	the cluster list with a complexity of n.log(n).
	 */
	public class ListenerSortCluster implements MouseListener, Runnable {  	  
		int column;
		Thread sorter;
				
		public void mousePressed(MouseEvent e) {	
			if (e.getClickCount() < 2) 
				return;				
			if (sorter != null && sorter.isAlive())
				return;			
			
			column = ((JTableHeader)e.getSource()).columnAtPoint(e.getPoint());
			if (column == 0)
				return;				
			statusBar.setTemporaryInfo(
		  	" Please wait while sorting selected ClusterSet data...", 200000);				
			sorter = new Thread(this);
			sorter.start();
		}
		
		public void mouseExited(MouseEvent e) { }      
	  public void mouseClicked(MouseEvent e) { }
	  public void mouseReleased(MouseEvent e) { }
	  public void mouseEntered(MouseEvent e) { }
	  
	  public void run() {
	  	try {
	  		ClientClusterSet ccsTmp = tableCluster.getClusterSet(); 	
	  		if (ccsTmp.getNbCluster() > 0)
					tableCluster.fusionSort(
						ccsTmp.getClusterList(), 0, ccsTmp.getNbCluster()-1, column);
				else {
					statusBar.setTemporaryInfo(" ERROR : No data to sort !", 3000);
					return;
				}

  			if (ccsTmp != null)
  				tableCluster.setClusterSet(ccsTmp); 				
	  		tableCluster.setSavable(true);
	  		statusBar.setTemporaryInfo(" ClusterSet data sorted !", 3000);
	  		repaint();
	  		
		  }
		  catch(Exception e) {
		  	PrintException(e);
		  	statusBar.setTemporaryInfo(
	  			" ERROR : An error occured while sorting data !", 3000);
		  }	  	
	  }
	}
	
		/**
	 * This listener manage the sorting job of the EST table.
	 * This is run by a 'double click' on the header of the column to sort.
	 * The sorting method implemented is the 'fusion sort' which sort 
	 *	the cluster list with a complexity of n.log(n).
	 */	
	public class ListenerSortEST implements MouseListener, Runnable {  	  		
		int column;
		Thread sorter;
				
		public void mousePressed(MouseEvent e) {	
			if (e.getClickCount() < 2) 
				return;				
			if (sorter != null && sorter.isAlive())
				return;
			
			column = ((JTableHeader)e.getSource()).columnAtPoint(e.getPoint());
			if (column == 0)
				return;				
			statusBar.setTemporaryInfo(
	  		" Please wait while sorting selected ESTSet data ...", 200000);				
			sorter = new Thread(this);
			sorter.start();
		}
		
		public void mouseExited(MouseEvent e) { }      
	  public void mouseClicked(MouseEvent e) { }
	  public void mouseReleased(MouseEvent e) { }
	  public void mouseEntered(MouseEvent e) { }
	  
	  public void run() {
	  	try {
	  		ClientESTSet estTmp = tableEST.getESTSet(); 	
	  		if (estTmp.getNbEST() > 0)
					tableEST.fusionSort(
						estTmp.getESTList(), 0, estTmp.getNbEST()-1, column);
				else {
					statusBar.setTemporaryInfo(" ERROR : No data to sort !", 3000);
					return;
				}

  			if (estTmp != null)
  				tableEST.setESTSet(estTmp); 				
	  		statusBar.setTemporaryInfo(" ESTSet data sorted !", 3000);
	  		repaint();
	  		
		  }
		  catch(Exception e) {
		  	PrintException(e);
		  	statusBar.setTemporaryInfo(
	  			" ERROR : An error occured while sorting data !", 3000);
		  }	  	
		}
	}
	
	
	/**
	 * This listener start sorting the GO table column's 
	 *	which was double-clicked.
	 */	
	public class ListenerSortGO implements MouseListener, Runnable {  	  		
		int column;
		Thread sorter;
				
		public void mousePressed(MouseEvent e) {	
			if (e.getClickCount() < 2) 
				return;				
			if (sorter != null && sorter.isAlive())
				return;
			
			column = ((JTableHeader)e.getSource()).columnAtPoint(e.getPoint());
			if (column == 0)
				return;				
			statusBar.setTemporaryInfo(
	  		" Please wait while sorting selected GOSet data ...", 200000);							
			sorter = new Thread(this);
			sorter.start();
		}
		
		public void mouseExited(MouseEvent e) { }      
	  public void mouseClicked(MouseEvent e) { }
	  public void mouseReleased(MouseEvent e) { }
	  public void mouseEntered(MouseEvent e) { }
	  
	  public void run() {
	  	try {
	  		ClientGOSet goTmp = tableGO.getGOSet(); 	
	  		if (goTmp.getNbGOFunctions() > 0)
					tableGO.fusionSort(
						goTmp.getGOList(), 0, goTmp.getNbGOFunctions()-1, column);
				else {
					statusBar.setTemporaryInfo(
	  			" ERROR : No data to sort !", 3000);
					return;
				}

  			if (goTmp != null)
  				tableGO.setGOSet(goTmp); 				
	  		statusBar.setTemporaryInfo(
	  			" GOSet data sorted !", 3000);
	  		repaint();
	  		
		  }
		  catch(Exception e) {
		  	PrintException(e);
		  	statusBar.setTemporaryInfo(
	  			" ERROR : An error occured while sorting data !", 3000);
		  }	  	
		}
	}
	
	/**
	 * The ListenerExportTXT's class is used to manage the export 
	 *	of a selected cluster data in a text file (*.txt).
	 */
	public class ListenerExportTXT implements ActionListener, Runnable {  
	  Thread exporter;
	  
	  public void actionPerformed(ActionEvent event) {
	  	if (exporter != null && exporter.isAlive())
	  		return;
	  	else {
	  		exporter = new Thread(this);
	  		exporter.start();
	  	}
	  }	
	  	
	  public void run() {	
	  	try {
				int sel = tableCluster.getSelectedRow();
				if (sel > -1)	{			
					String clu = (String)tableCluster.getValueAt(sel, 2);	
					String gene = (String)tableCluster.getValueAt(sel, 3);
					statusBar.setTemporaryInfo(
	  				" Please wait while exporting informations about "+clu+"..."
	  			, 1000000);
					
					BufferedWriter out = new BufferedWriter(new FileWriter(clu+".txt"));
					String toWrite = "TIGR Cluster: "+ clu + "\n";
					out.write(toWrite);
					toWrite = "NCBI Unigene: "+ gene + "\n\n";
					out.write(toWrite);
					toWrite = "";
					
					ClientCluster cctmp = tableCluster.getClusterSet().getCluster(sel);					
					ClientESTSet estSetTmp = tableEST.getESTSet();					
					ClientEST estTmp;
					for (int i = 0; i < estSetTmp.getNbEST(); ++i) {
						estTmp = estSetTmp.getEST(i);
						toWrite += estTmp.getESTCode() + "\t\t";						
						toWrite += tableEST.HTitle.get(""+estTmp.getLibrary()) + "\t\t";
						toWrite += tableEST.HTissue.get(""+estTmp.getTissue()) + "\t\t";
						toWrite += tableEST.HDevStage.get(""+estTmp.getDevStage()) + "\n";
						out.write(toWrite);
						toWrite = "";
					}
					out.write("\n\n");
					
					ClientGOSet goSetTmp = tableGO.getGOSet();
					ClientGO goTmp;	
					for (int i = 0; i < goSetTmp.getNbGOFunctions(); ++i) {
						goTmp = goSetTmp.getGOFunction(i);
						toWrite += goTmp.getGOCode() + "\t\t";
						toWrite += goTmp.getGOTerm() + "\t\t";
						toWrite += goTmp.getGOType() + "\t\t";
						toWrite += goTmp.getGOpep() + "\n";
						out.write(toWrite);
						toWrite = "";
					}
					out.close();
					statusBar.setTemporaryInfo(
	  				" Informations about "+clu+" saved in "+clu+".txt !", 3000);
				}
				else {
					statusBar.setTemporaryInfo(
	  				" ERROR : No cluster selected to export !", 3000);
				}
			}
			catch (Exception e) {
				PrintException(e);
			}
		}
	}
	
		/**
	 * The ListenerExportCSV's class is used to manage the export 
	 *	of a selected cluster data in a text file (*.csv).
	 */
	public class ListenerExportCSV implements ActionListener, Runnable {  
	  Thread exporter;
	  
	  public void actionPerformed(ActionEvent event) {
	  	if (exporter != null && exporter.isAlive())
	  		return;
	  	else {
	  		exporter = new Thread(this);
	  		exporter.start();
	  	}
	  }	
	  	
	  public void run() {	
	  	try {
				int sel = tableCluster.getSelectedRow();
				if (sel > -1)	{			
					String clu = (String)tableCluster.getValueAt(sel, 2);
					String gene = (String)tableCluster.getValueAt(sel, 3);
					statusBar.setTemporaryInfo(
	  				" Please wait while exporting informations about "+clu+"..."
	  			, 1000000);
					
					PrintWriter out = new PrintWriter(new FileWriter(clu+".csv"));
    			ClientCSVWriter csv = new ClientCSVWriter(out);
					
					csv.writeField("TIGR Cluster: " + clu);
					csv.writeField("NCBI Unigene: " + gene);
					csv.endLine();
					csv.endLine();
					
					ClientCluster cctmp = tableCluster.getClusterSet().getCluster(sel);					
					ClientESTSet estSetTmp = tableEST.getESTSet();					
					ClientEST estTmp;
					for (int i = 0; i < estSetTmp.getNbEST(); ++i) {
						estTmp = estSetTmp.getEST(i);
						csv.writeField(estTmp.getESTCode());
						csv.endBlock();						
						csv.writeField((String)tableEST.HTitle.get(""+estTmp.getLibrary()));
						csv.endBlock();									
						csv.writeField((String)tableEST.HTissue.get(""+estTmp.getTissue()));
						csv.endBlock();	
						csv.writeField((String)tableEST.HDevStage.get(""+estTmp.getDevStage()));
					}
					csv.endLine();
					csv.endLine();
					
					ClientGOSet goSetTmp = tableGO.getGOSet();
					ClientGO goTmp;	
					for (int i = 0; i < goSetTmp.getNbGOFunctions(); ++i) {
						goTmp = goSetTmp.getGOFunction(i);
						csv.writeField(""+goTmp.getGOCode());
						csv.endBlock();	
						csv.writeField(goTmp.getGOTerm());
						csv.endBlock();	
						csv.writeField(""+goTmp.getGOType());
						csv.endBlock();	
						csv.writeField(goTmp.getGOpep());
					}
					csv.endLine();
					csv.close();
					out.close();
					
					statusBar.setTemporaryInfo(
	  				" Informations about "+clu+" saved in "+clu+".csv !", 3000);
				}
				else {
					statusBar.setTemporaryInfo(
	  				" ERROR : No cluster selected to export !", 3000);
				}
			}
			catch (Exception e) {
				PrintException(e);
			}
		}
	}
	
	/**
	 * The ListenerClusterTIGRInfo's class is used 
	 *	to show additionnal informations about the selected cluster
	 *	which are present on the TIGR TGI web site.
	 */
	public class ListenerClusterTIGRInfo implements ActionListener, Runnable {  
	  Thread viewer;
	  
	  public void actionPerformed(ActionEvent event) {
	  	if (viewer != null && viewer.isAlive())
	  		return;
	  	else {
	  		statusBar.setTemporaryInfo(
	  			" Please wait while connecting to TIGR TGI..."
	  		, 3000);  			
	  		viewer = new Thread(this);
	  		viewer.start();
	  	}
	  }	
	  	
	  public void run() {	
	  	try {
	  		String url = "http://www.tigr.org/tigr-scripts/tgi/tc_report.pl?";
				int sel = tableCluster.getSelectedRow();
				if (sel > -1)	{			
					String search = csp.getSpecieSearch();
					String cluster = (String)tableCluster.getValueAt(sel, 2);	
	  			String specie = (String)cp.TC_REPORT_MAP.get(search);
					url += "species=" + specie + "&tc=" + cluster + "&display=1";
					String title = " TIGR ["+search+" - "+cluster+"] ";
					
					ClientViewer cv;
					int nbEST = Integer.parseInt((String)tableCluster.getValueAt(sel, 6));		
					if (nbEST <= 500) {
						cv = new ClientViewer(url, cp);
						tabbedPane.addTab(
							title, new ImageIcon("./ico/ico-network.png"), cv, title);
						tabbedPane.setSelectedComponent(cv);
					}
					else {						
						cv = new ClientViewer("#" + url, cp);
						tabbedPane.addTab(
							title, new ImageIcon("./ico/ico-network.png"), cv, title);
						tabbedPane.setSelectedComponent(cv);
						connectionTooMuchAlignFrame(ClientGUI.this);						
					}					
				}
				else {
					statusBar.setTemporaryInfo(
	  				" ERROR : No cluster selected to browse in TIGR TGI !", 3000);
				}
			}
			catch (Exception e) {
				PrintException(e);
			}
		}
	}
	
	/**
	 * The ListenerUnigeneNCBIInfo's class is used 
	 *	to show additionnal informations about the selected gene
	 *	which are present on the NCBI Unigene web site.
	 */
	public class ListenerUnigeneNCBIInfo implements ActionListener, Runnable {  
	  Thread viewer;
	  
	  public void actionPerformed(ActionEvent event) {
	  	if (viewer != null && viewer.isAlive())
	  		return;
	  	else {		  			
	  		viewer = new Thread(this);
	  		viewer.start();
	  	}
	  }	
	  	
	  public void run() {	
	  	try {
	  		String url = "http://www.ncbi.nlm.nih.gov/UniGene/clust.cgi?";
				int sel = tableCluster.getSelectedRow();
				if (sel > -1)	{			
					String search = csp.getSpecieSearch();
					String gene = (String)tableCluster.getValueAt(sel, 3);	
					
					if (gene.contentEquals(new StringBuffer("---"))) {
						statusBar.setTemporaryInfo(
	  					" ERROR : This cluster is not a Unigene gene in NCBI Unigene !"
	  				, 3000);						
						return;
					}
					statusBar.setTemporaryInfo(
	  			" Please wait while connecting to NCBI UniGene..."
	  			, 3000);  	
	  			
					String geneCode = gene.substring(gene.lastIndexOf(".")+1);
					url += "ORG=" + search + "&CID=" + geneCode + "&OPT=text";
					String title = " UniGene ["+search+" - "+gene+"] ";
					
					ClientViewer cv = new ClientViewer(url, cp);
					tabbedPane.addTab(
						title, new ImageIcon("./ico/ico-network.png"), cv, title);
					tabbedPane.setSelectedComponent(cv);
				}
				else {
					statusBar.setTemporaryInfo(
	  				" ERROR : No cluster selected to browse in NCBI UniGene !", 3000);
				}
			}
			catch (Exception e) {
				PrintException(e);
			}
		}
	}
	
	/**
	 * The ListenerESTTIGRInfo's class is used 
	 *	to show additionnal informations about the selected EST
	 *	which are present on the TIGR TGI web site.
	 */
	public class ListenerESTTIGRInfo implements ActionListener, Runnable {  
	  Thread viewer;
	  
	  public void actionPerformed(ActionEvent event) {
	  	if (viewer != null && viewer.isAlive())
	  		return;
	  	else if (!showESTInfo.isSelected()) {
	  		statusBar.setTemporaryInfo(
	  			" ERROR : No EST selected to browse on TIGR TGI !", 3000);
	  		return;
	  	}	  	
	  	else {
	  		statusBar.setTemporaryInfo(
	  			" Please wait while connecting to TIGR TGI..."
	  		, 3000);  			
	  		viewer = new Thread(this);
	  		viewer.start();	  		
	  	}	  	
	  }	
	  	
	  public void run() {	
	  	try {
	  		String url = "http://www.tigr.org/tigr-scripts/tgi/est_report.pl?";
				int sel = tableEST.getSelectedRow();
				if (sel > -1)	{			
					String search = csp.getSpecieSearch();
					String est = tableEST.getValueAt(sel, 1);	
	  			String specie = (String)cp.TC_REPORT_MAP.get(search);
					url += "species=" + specie + "&EST=" + est;
					String title = " TGI ["+search+" - "+est+"] ";					

					ClientViewer cv = new ClientViewer(url, cp);
					tabbedPane.addTab(
						title, new ImageIcon("./ico/ico-network.png"), cv, title);
					tabbedPane.setSelectedComponent(cv);
				}
				else {
					statusBar.setTemporaryInfo(
	  				" ERROR : No EST selected to browse on TIGR TGI !", 3000);
				}
			}
			catch (Exception e) {
				PrintException(e);
			}
		}
	}
	
	/**
	 * The ListenerAmiGOInfo's class is used 
	 *	to show additionnal informations about the selected Go-Function
	 *	which are present on the Swiss-Prot ExPaSy web site.
	 */
	public class ListenerAmiGOInfo implements ActionListener, Runnable {  
	  Thread viewer;
	  
	  public void actionPerformed(ActionEvent event) {
	  	if (viewer != null && viewer.isAlive())
	  		return;
	  	else if (!showGOInfo.isSelected()) {
	  		statusBar.setTemporaryInfo(
	  			" ERROR : No Ontology term selected to browse on AmiGO !", 3000);
	  		return;
	  	}	 
	  	else {
	  		statusBar.setTemporaryInfo(
	  			" Please wait while connecting to AmiGO..."
	  		, 3000); 			
	  		viewer = new Thread(this);
	  		viewer.start();	  		
	  	}	  	
	  }	
	  	
	  public void run() {	
	  	try {
	  		String url = "http://www.godatabase.org/cgi-bin/amigo/go.cgi?format=go_ff&query=";
				int sel = tableGO.getSelectedRow();
				if (sel > -1)	{			
					String search = csp.getSpecieSearch();
					String go = tableGO.getValueAt(sel, 1);	
					String title = " AmiGO ["+search+" - "+go+"] ";					
					
					ClientViewer cv = new ClientViewer(url+go, cp);
					tabbedPane.addTab(
						title, new ImageIcon("./ico/ico-network.png"), cv, title);
					tabbedPane.setSelectedComponent(cv);
				}
				else {
					statusBar.setTemporaryInfo(
	  				" ERROR : No Ontology term selected to browse on AmiGO !", 3000);
				}
			}
			catch (Exception e) {
				PrintException(e);
			}
		}
	}
	
	/**
	 * The ListenerExPASyInfo's class is used 
	 *	to show additionnal sequence about the selected protein
	 *	which are present on the Swiss-Prot ExPaSy web site.
	 */
	public class ListenerExpasyInfo implements ActionListener, Runnable {  
	  Thread viewer;
	  
	  public void actionPerformed(ActionEvent event) {
	  	if (viewer != null && viewer.isAlive())
	  		return;
	  	else if (!showGOInfo.isSelected()) {
	  		statusBar.setTemporaryInfo(
	  			" ERROR : No Protein selected to browse on Swiss-Prot ExPASy !", 3000);
	  		return;
	  	}	
	  	else {	
	  		statusBar.setTemporaryInfo(
	  			" Please wait while connecting to ExPASy..."
	  		, 3000); 				
	  		viewer = new Thread(this);
	  		viewer.start();	  		
	  	}	  	
	  }	
	  	
	  public void run() {	
	  	try {
	  		//String url = "http://au.expasy.org/cgi-bin/get-sprot-fasta?";
	  		String url = "http://au.expasy.org/cgi-bin/niceprot.pl?";

				int sel = tableGO.getSelectedRow();
				if (sel > -1)	{			
					String search = csp.getSpecieSearch();
					String gopep = tableGO.getValueAt(sel, 4);
					
					if (gopep.startsWith("GOA_SPTR|")) {
						gopep = gopep.substring(9);
						gopep = gopep.substring(0, 6);
					}
					else if (gopep.startsWith("GOA_human|")) {
						gopep = gopep.substring(10);
						gopep = gopep.substring(0, 6);
					}
					else {
						statusBar.setTemporaryInfo(
	  					" ERROR : Not linked in Swiss-Prot ExPASy !"
	  				, 3000);						
						return;
					}
					statusBar.setTemporaryInfo(
	  			" Please wait while connecting to Swiss-Prot ExPASy..."
	  		, 3000);
						
					String title = " ExPasY ["+search+" - "+gopep+"] ";					
					
					ClientViewer cv = new ClientViewer(url+gopep, cp);
					tabbedPane.addTab(
						title, new ImageIcon("./ico/ico-network.png"), cv, title);
					tabbedPane.setSelectedComponent(cv);
				}
				else {
					statusBar.setTemporaryInfo(
	  				" ERROR : No Protein selected to browse on Swiss-Prot ExPASy !"
	  			, 3000);
				}
			}
			catch (Exception e) {
				PrintException(e);
			}
		}
	}
	
		
	/**
	 * The class ListenerClickCluster is a listener used to check the component clicked
	 *	and then do the appropriate operation.
	 */	
	public class ListenerClickCluster implements MouseListener, Runnable {  	  
		Thread updater;
		
		public void mousePressed(MouseEvent e) {	
			tableEST.setESTSet(new ClientESTSet(0));	
	  	tableGO.setGOSet(new ClientGOSet(0));	
	  	repaint();
	  		
			if (updater != null && updater.isAlive())
				return;
			else {						
				updater = new Thread(this);
				updater.start();
			}				
		}
		
	  public void mouseExited(MouseEvent e) { }      
	  public void mouseClicked(MouseEvent e) { }
	  public void mouseReleased(MouseEvent e) { }
	  public void mouseEntered(MouseEvent e) { }
	  
	  public void run() {
	  	try {
	  		String cluster;
				int parsedCluster;
				
	  		int sel = tableCluster.getSelectedRow();				
				String tmpClu = (String)tableCluster.getValueAt(sel, 2);
								
				if (showESTInfo.isSelected() || showGOInfo.isSelected())
					statusBar.setTemporaryInfo(
						" Please wait while getting informations about "+tmpClu+"...", 
					1000000);	
			
				if (csp.getSpecieSearch().startsWith("Hs")) 
					cluster = tmpClu.substring(3);
				else cluster = tmpClu.substring(2);
				parsedCluster = Integer.parseInt(cluster);
	  			  		
				if (showGOInfo.isSelected()) {																
					if (ctrgo != null) {
						if (ctrgo.isAlive())
							ctrgo.stop();	
					}
														
					ctrgo = new ClientThreadRetrieveGO(
						csp.getSpecieSearch(), parsedCluster, new ClientRequestGO(con, cp));					
					ctrgo.start();
					if (ctrgo != null)
						ctrgo.join();				
				}		
					
				if (showESTInfo.isSelected()) {						
					if (ctrest != null) {
						if (ctrest.isAlive())
							ctrest.stop();						
					}					
					ctrest = new ClientThreadRetrieveEST(
						csp.getSpecieSearch(), parsedCluster, new ClientRequestEST(con, cp));								
					ctrest.start();
				}
				
				
				if (ctrest != null)					
					ctrest.join();

				if (showESTInfo.isSelected() && ctrest != null) {			
					tableEST.setESTSet(ctrest.getESTSet());
					statusBar.setTemporaryInfo(
			  		" Informations about selected cluster collected !", 3000);
				}	
						  		 				
				if (showGOInfo.isSelected() && ctrgo != null) {
					tableGO.setGOSet(ctrgo.getGOSet());	
			  	statusBar.setTemporaryInfo(
			  		" Informations about selected cluster collected !", 3000);
			  }
			  
			  if (ctrest == null && ctrgo == null)		
		  		statusBar.setTemporaryInfo(
			  		" Ready !", 3000);
			  repaint(); 
			}
			catch (Exception e) {
				PrintException(e);
			}													  	
	  }
	}

	/**
	 * The class ListenerShowESTInfo is a listener used to display
	 *	EST informations panel.
	 */
	public class ListenerShowESTInfo implements ActionListener, Runnable {  
	  public void actionPerformed(ActionEvent event) {
	  	try {
	  		if (showESTInfo.isSelected()) {
	  			int sel = tableCluster.getSelectedRow();				
					String cluster = (String)tableCluster.getValueAt(sel, 2);
					if (csp.getSpecieSearch().startsWith("Hs")) 
						cluster = cluster.substring(3);
					else cluster = cluster.substring(2);
					int parsedCluster = Integer.parseInt(cluster);
					if (ctrest != null) {
						if (ctrest.isAlive())
							ctrest.stop();						
					}					
					ctrest = new ClientThreadRetrieveEST(
						csp.getSpecieSearch(), parsedCluster, new ClientRequestEST(con, cp));								
					ctrest.start();
					tableEST.setESTSet(new ClientESTSet(0));
					repaint();
	  			new Thread(this).start();	  			
	  		}
	  		else {
	  			tableEST.setESTSet(new ClientESTSet(0));
					panelCenter.remove(tableEST);			
	  			refreshPanelSize();
	  			repaint();
	  		}						
	  	}	
	  	catch (Exception e) {
	  		tableEST.setESTSet(new ClientESTSet(0));
				panelCenter.add(tableEST, BorderLayout.CENTER);			  		
	  		refreshPanelSize();
	  		repaint();
	  	}
	  }
	  
	  public void run() {
	  	try {
				ctrest.join();
				tableEST.setESTSet(ctrest.getESTSet()); 
		  	panelCenter.add(tableEST, BorderLayout.CENTER);
		  	refreshPanelSize();
		  	repaint();
		  }
		  catch (Exception e) {
		  	PrintException(e);
		  }
	  }
	}
	
	/**
	 * The class ListenerShowGOInfo is a listener used to display
	 *	Go-Functions informations panel.
	 */
	public class ListenerShowGOInfo implements ActionListener, Runnable {  
	  public void actionPerformed(ActionEvent event) {
	  	try {
	  		if (showGOInfo.isSelected()) {
	  			int sel = tableCluster.getSelectedRow();				
					String cluster = (String)tableCluster.getValueAt(sel, 2);
					if (csp.getSpecieSearch().startsWith("Hs")) 
						cluster = cluster.substring(3);
					else cluster = cluster.substring(2);
					int parsedCluster = Integer.parseInt(cluster);
					if (ctrgo != null) {
						if (ctrgo.isAlive())
							ctrgo.stop();	
					}									
					ctrgo = new ClientThreadRetrieveGO(
						csp.getSpecieSearch(), parsedCluster, new ClientRequestGO(con, cp));					
					ctrgo.start();
					tableGO.setGOSet(new ClientGOSet(0)); 
					repaint();
	  			new Thread(this).start();
	  		}
	  		else {
	  			tableGO.setGOSet(new ClientGOSet(0)); 		  			
	  			panelCenter.remove(tableGO);
	  			refreshPanelSize();
	  			repaint();
	  		}							
	  	}	
	  	catch (Exception e) {
	  		tableGO.setGOSet(new ClientGOSet(0)); 		  			
	  		panelCenter.add(tableGO, BorderLayout.SOUTH);
	  		refreshPanelSize();
	  		repaint();
		  }
	  }
	  
	  public void run() {
	  	try {
				ctrgo.join();
				tableGO.setGOSet(ctrgo.getGOSet());
				panelCenter.add(tableGO, BorderLayout.SOUTH);
		  	refreshPanelSize();
		  	repaint();
	  	}
		  catch (Exception e) {
		  	PrintException(e);
		  }				
	  }
	}
	
	/**
	 * The class ListenerButtonSave is the listener used to save
	 *	the result of a search.
	 */
 	public class ListenerButtonSave implements ActionListener, Runnable {
   	public void actionPerformed(ActionEvent event) {   	
    	new Thread(this).start();
    }
    
    public void run() {
    	if (cts != null && cts.isAlive() 
    		|| tableCluster.getClusterSet().getNbCluster() == 0) {
    		statusBar.setTemporaryInfo(
    			" ERROR : Nothing to save at this time !", 3000);
    		try { Thread.sleep(2000);	}
    		catch (Exception e) { }
    		if (cts != null && cts.isAlive())
    			statusBar.setTemporaryInfo(
		    		" Please wait while computing your search...", 10000000);	
    	}
    	else {   		
	    	save(tableCluster.getClusterSet());
	     	tableCluster.setSavable(false);
     	}
      repaint();
    }
  }
	
	/**
	 * The class ListenerButtonLoad is the listener used to load
	 *	the result of the current search.
	 */
	public class ListenerButtonLoad implements ActionListener {
	  public void actionPerformed(ActionEvent event) {
	    load();
	    repaint();
	  }
	}
	
	/**
	 * The class ListenerButtonExit is the listener used to 
	 *	close 'softly' the application.
	 */
  public class ListenerButtonExit implements ActionListener, Runnable {
    public void actionPerformed(ActionEvent event) {
			new Thread(this).start();
		}
		
		public void run() {	
			if (ctrest != null && ctrest.isAlive())
				ctrest.stop();
			if (ctrgo != null && ctrgo.isAlive())
				ctrgo.stop();
			
			if (cts != null && cts.isAlive()) {
	  		if (confirmCancelSearch(ClientGUI.this)) {
	  			cts.stop();
    			WindowEvent we = new WindowEvent(ClientGUI.this,0);
      		closeFrame.windowClosing(we);
    		}
    	}
    	else {
    		WindowEvent we = new WindowEvent(ClientGUI.this,0);
      	closeFrame.windowClosing(we);
    	}
    }
  }
  
  /**
	 * The class ListenerButtonAbout is the listener used to 
	 *	show development and version informations.
	 */
  public class ListenerButtonAbout implements ActionListener, Runnable {
    public void actionPerformed(ActionEvent event) {
			new Thread(this).start();
		}
		
		public void run() {
			ClientFrameAbout cfa = new ClientFrameAbout();
			statusBar.setTemporaryInfo(
			  " bioECS 1.0  -  Created by Benjamin Renwart", 3000);
			try { Thread.sleep(2500); }
			catch (Exception e) { }
			statusBar.setTemporaryInfo(
			  " Universit de Lige  -  Ulg (2004/2005)", 3000);
			try { Thread.sleep(2500); }
			catch (Exception e) { }
			if (cts != null && cts.isAlive())
				statusBar.setTemporaryInfo(
		    	" Please wait while computing your search...", 10000000);	
		  repaint();
    }
  }
  
  /**
	 * The class ListenerButtonSearchAbort is the listener used to 
	 *	abort the current search processus.
	 */
  public class ListenerButtonSearchAbort implements ActionListener, Runnable {
    public void actionPerformed(ActionEvent event) {
			new Thread(this).start();
		}
		
		public void run() {
			if (cts != null && cts.isAlive()) {
	  		if (confirmCancelSearch(ClientGUI.this)) {
	  			cts.stop();
	  			tabbedPane.setEnabledAt(1, true);  
    			stoppedSearch = true; 		
    		}
    	}
			else {
    		statusBar.setTemporaryInfo(
  				" ERROR : No running search to abort !", 3000);	
  		}
  	} 
	}

	
	/***************************************************************************/
	
	
	
  /**
   * The ClientFrameCloser's class allows a 'soft' close
   *	of the ClientGUI frame.
   */
  public class ClientFrameCloser extends WindowAdapter implements Runnable {
		/**
		 * Method which manage the frame closing by telling user if
		 *	he wants to save his search and if he is sure to exit the program.
		 * @param the window event associated with a frame closing.
		 */		
		public void windowClosing(WindowEvent event) {
      new Thread(this).start();
    }
    
    public void run() {
    	try {
	    	if (confirmQuit(ClientGUI.this)) {
	        if (tableCluster.getSavable() && confirmSave(ClientGUI.this)) {
	          if (dir.equalsIgnoreCase("")) {
	            save(tableCluster.getClusterSet());
	            tableCluster.setSavable(false);
	            repaint();
	          }
	          else {
	            ClientClusterSet.saveClusterSet(tableCluster.getClusterSet(),dir);
	          }
	        }
	        // Delete tables created by client in this session before exiting.
	        crm.deleteClientTables(clientIP);        							  
	        //close the data base connection.
	        crm.closeConnection();
	        // finally, exit the program.
	        System.exit(0);
	      }   
	    }
	    catch (Exception e) {
		  	PrintException(e);
		  } 	
    }
    
		/**
		 * Show a confirm saving frame telling user to save or not.
		 * @param frame the parent frame associated.
		 * @return a boolean indicating the user choice to save or not.
		 */
    private boolean confirmSave(JFrame frame) {
      String s1 = "Yes";
      String s2 = "No";
      Object[] options = {s1, s2};
      int n = JOptionPane.showOptionDialog(frame,
      	"Do you want to save your search ?","Save ?",JOptionPane.YES_NO_OPTION, 
      	JOptionPane.QUESTION_MESSAGE, new ImageIcon("./ico/bioecs.png"), 
      	options, s1);
      if (n == JOptionPane.YES_OPTION) return true;
      else return false;
    }
		
		/**
		 * Show a confirm exiting frame telling user to exit or not.
		 * @param frame the parent frame associated.
		 * @return a boolean indicating the user choice to exit or not.
		 */
   	private boolean confirmQuit(JFrame frame) {
      String s1 = "Quit";
      String s2 = "Cancel";
      Object[] options = {s1, s2};
      int n = JOptionPane.showOptionDialog(frame, 
      	"Do you really want to exit ?", "Quit ?", JOptionPane.YES_NO_OPTION, 
      	JOptionPane.QUESTION_MESSAGE, new ImageIcon("./ico/bioecs.png"), 
      	options, s1);      
      if (n == JOptionPane.YES_OPTION) return true;
      else return false;
    }
  }
   
  /** 
   * The ClientFrameSearch is the frame used to manage the search parameters.
   * This frame allows clients to select a specie and parameters for a search.
   * When the user has selected its parameters, he can run the search thread.
   */
  private class ClientFrameSearch extends JPanel {					

		private JPanel sp;		
		private JComboBox specieBox;
		private JTextArea specieInfo;
		private JScrollPane scrInfo;
		private String specieInfoText;
		private JButton oks;
		private JButton exportAssocTXT;
		private JButton exportAssocCSV;
				
		private JPanel op; 
		private JPanel options;
		private JScrollPane scrPanels;
		private JPanel info;
		private JPanel command;
		private JButton oko;
		private JButton cancelo;
		private JPanel add;
		private JButton addT;
		private JButton addD;

		private String specieCode;
		private int selectedSpecie;		
		private Object[] specieToShow;
	  private ArrayList panelList;
	  private int numCriteria;	  
	  private ClientSearchParameterSet lastSearch;

		/**
		 * Construct a new ClientFrameSearch object.
		 * @param species the specie installed in the data base
		 */
	  public ClientFrameSearch(Object[] species) {    
	    this.specieToShow = species;
	    this.numCriteria = 0;
	    this.panelList = new ArrayList();
	    this.lastSearch = null;
	    
	    this.loadAutoResizer();
	    this.buildSpeciePanel(false);
	    this.setForeground(Color.darkGray);		
			this.setLayout(new BorderLayout());
		  this.add(sp, BorderLayout.CENTER);			  			
	    this.setVisible(true);	    
	  }
	  
	  /**
		 * Construct a new ClientFrameSearch object.
		 * @param csps a predefined search parameter set.
		 */
	  public ClientFrameSearch(ClientSearchParameterSet csps) {    
	    csp = csps;
	    this.lastSearch = csps;
	    this.numCriteria = csp.getNbParameter()+1;
	    this.panelList = new ArrayList();
	    
	    tableEST.setESTSet(new ClientESTSet(0));
	  	tableGO.setGOSet(new ClientGOSet(0));
			
			// update the content of HashMaps according to the specie
			tableEST.HTissue = crm.get_code_converter_tissue(csp.getSpecieSearch());					
	  	tableEST.HDevStage = crm.get_code_converter_dev_stage(csp.getSpecieSearch());	
	  	tableEST.HTitle = crm.get_code_converter_title(csp.getSpecieSearch());	  		
	  	HTissueWage = crm.get_wage_tissue(csp.getSpecieSearch());				
			HDevStageWage = crm.get_wage_dev_stage(csp.getSpecieSearch());		
	    
	    if (cp.INSTALLED_SPECIES != null) {
		    specieToShow = new Object[cp.INSTALLED_SPECIES.length];   	
	    	for (int i = 0; i < cp.INSTALLED_SPECIES.length; ++i) {
	    		specieToShow[i] = cp.INSTALLED_SPECIES[i][6].replace('_', ' ');
	    		if (cp.INSTALLED_SPECIES[i][0].equals(csp.getSpecieSearch())) {
	    			selectedSpecie = i;
	    			specieCode = (String)cp.INSTALLED_SPECIES[selectedSpecie][0];
	    		}
	    	} 
	    }  	
    	
    	this.loadAutoResizer();    	 	
	    this.buildSpeciePanel(true);
			this.buildSearchOptionsPanel();
			this.buildSearchPanels(csp);  
	    this.setForeground(Color.darkGray);		
			this.setLayout(new BorderLayout());
		  this.add(op, BorderLayout.CENTER);			  			
	    this.setVisible(true);	    
	  }

		/**
		 * Build the specie selection panel allowing the client to select a specie.
		 * @loaded is true if it's a loaded search (previously saved) else false
		 */
	  private void buildSpeciePanel(boolean loaded) { 	
	  	JLabel specie = new JLabel("Selected specie : ");
	    specieBox = new JComboBox(specieToShow);   
	    	    
	    if (!loaded) {
	    	selectedSpecie = specieBox.getSelectedIndex();
				specieCode = (String)cp.INSTALLED_SPECIES[selectedSpecie][0];
			}
			else specieBox.setSelectedItem(specieToShow[selectedSpecie]);
			specieBox.addActionListener(new ListSpecieBoxChange());

	    oks = new JButton("  OK    ", new ImageIcon("./ico/ico-check.gif"));	
	    oks.addActionListener(new ListOKs());
	    
	    
	    exportAssocTXT = new JButton(
	    	" Export in *.txt ", new ImageIcon("./ico/ico-export-assoc-txt.png"));	
	    exportAssocTXT.addActionListener(new ListenerExportAssocTXT());
	    
	    exportAssocCSV = new JButton(
	    	" Export in *.csv ", new ImageIcon("./ico/ico-export-assoc-csv.png"));	
	    exportAssocCSV.addActionListener(new ListenerExportAssocCSV());
	    
	    JPanel export = new JPanel();
	  	export.setForeground(Color.darkGray);
	  	export.setBorder(BorderFactory.createTitledBorder(
		  	"Cluster's Association : UniGene / TGI")); 			
			export.setLayout(new GridBagLayout());			
		  addComponent(export, exportAssocTXT, 0, 0, 1, 1, GridBagConstraints.EAST);
		  addComponent(export, exportAssocCSV, 1, 0, 1, 1, GridBagConstraints.WEST); 
	    
	  	JPanel specieInfoTop = new JPanel();
	  	specieInfoTop.setForeground(Color.darkGray);			
			specieInfoTop.setLayout(new GridBagLayout());			
		  addComponent(specieInfoTop, specie, 0, 0, 1, 1, GridBagConstraints.EAST);
		  addComponent(specieInfoTop, oks, 2, 0, 1, 1, GridBagConstraints.WEST); 
		 	addComponent(specieInfoTop, specieBox, 1, 0, 1, 1, GridBagConstraints.WEST);
		 	addComponent(specieInfoTop, export, 3, 0, 1, 1, GridBagConstraints.EAST);
		 	specieInfoTop.setVisible(true);		 	
			
			specieInfoText = crm.get_INSTALLED_SPECIE_INFO(specieCode);
			specieInfo = new JTextArea(specieInfoText);
			specieInfo.setEditable(false);
			scrInfo = new JScrollPane(specieInfo);			

	  	sp = new JPanel();	  
		  sp.setBorder(BorderFactory.createTitledBorder(
		  	"  SEARCH STEP 1  ::  Select a specie...            ")); 
			sp.setForeground(Color.darkGray);			
			sp.setLayout(new BorderLayout());			
		  sp.add(specieInfoTop, BorderLayout.NORTH);
		  sp.add(scrInfo, BorderLayout.CENTER);
		  
		  if (!loaded)
		  	sp.setVisible(true);
		  else sp.setVisible(false);
	  }
	  
	  /**
	   * Build the search option panel which allows the client 
	   *	to add the desired number of search criteria as he wants.
	   */
	  private void buildSearchOptionsPanel() {     
	  	this.buildInfoBoxPanel();
	  	this.buildCommandBoxPanel();
	  	this.buildAddBoxPanel();
	  	
	  	JPanel optionsTop = new JPanel();
	  	optionsTop.setForeground(Color.darkGray);			
			optionsTop.setLayout(new GridBagLayout());			
		  addComponent(optionsTop, info, 0, 0, 1, 1, GridBagConstraints.CENTER);
		  addComponent(optionsTop, add, 1, 0, 1, 1, GridBagConstraints.EAST); 
		 	addComponent(optionsTop, command, 2, 0, 1, 1, GridBagConstraints.WEST);		 	
	  	optionsTop.setVisible(true);
	  	
	  	options = new JPanel();
	  	options.setLayout(new GridBagLayout());
	  	scrPanels = new JScrollPane(options);	
	  	
	  	op = new JPanel();	  
		  op.setBorder(BorderFactory.createTitledBorder(
		  	"  SEARCH STEP 2  ::  Select your parameters...         ")); 
			op.setForeground(Color.darkGray);
			op.setLayout(new BorderLayout());			
		  op.add(optionsTop, BorderLayout.NORTH);
		  op.add(scrPanels, BorderLayout.CENTER);
		  op.setVisible(true);
		}
		
		/**
	   * Build the specie information box of the search option panel.
	   */
		private void buildInfoBoxPanel() {		
			JLabel specieLabel = new JLabel(
	    	(String)cp.INSTALLED_SPECIES[selectedSpecie][6].replace('_', ' '));
	    JLabel specieLabelNCBICode = new JLabel(
	    	"NCBI Code :  "+(String)cp.INSTALLED_SPECIES[selectedSpecie][0]);
	    JLabel specieLabelTIGRCode = new JLabel(
	    	"TIGR Code :  "+(String)cp.INSTALLED_SPECIES[selectedSpecie][3]);    
	    JLabel specieLabelNCBI = new JLabel(
	    	"       Installed NCBI Release :  "+
	    	(String)cp.INSTALLED_SPECIES[selectedSpecie][1]);
	    JLabel specieLabelTIGR = new JLabel(
	    	"       Installed TIGR Release :  "+
	    	(String)cp.INSTALLED_SPECIES[selectedSpecie][4]);
			
			info = new JPanel();	  
		  info.setBorder(BorderFactory.createTitledBorder(
		  	" Selected specie informations : ")); 
			info.setForeground(Color.darkGray);
			info.setLayout(new GridBagLayout());
		  addComponent(info, specieLabel, 0, 0, 1, 1);
		 	addComponent(info, specieLabelNCBICode, 0, 1, 1, 1);	
			addComponent(info, specieLabelNCBI, 1, 1, 1, 1); 	
			addComponent(info, specieLabelTIGRCode, 0, 2, 1, 1);	
			addComponent(info, specieLabelTIGR, 1, 2, 1, 1); 
		} 
		
		/**
	   * Build the command box of the search option panel.
	   */
		private void buildCommandBoxPanel() {
	    oko = new JButton(
	    	"      Run Search !    ", new ImageIcon("./ico/ico-go.png"));
	    oko.addActionListener(new ListOKo());   
	    cancelo = new JButton(
	    	" Select another specie", new ImageIcon("./ico/ico-del.png"));
	    cancelo.addActionListener(new ListCancelo());
	    
	    command = new JPanel();	  
		  command.setBorder(BorderFactory.createTitledBorder(
		  	" Command Box : ")); 
			command.setForeground(Color.darkGray);
			command.setLayout(new GridBagLayout());							
			addComponent(command, oko, 0, 0, 1, 1, GridBagConstraints.CENTER);	
			addComponent(command, cancelo, 0, 1, 1, 1, GridBagConstraints.CENTER);
		}
		
		/**
	   * Build the 'add criteria' box of the search option panel.
	   */
		private void buildAddBoxPanel() {
	    addD = new JButton(
	    	" Development stage", new ImageIcon("./ico/ico-frame.gif"));
	    addD.addActionListener(new ListAddDevStage()); 
	    addT = new JButton(
	    	"      Tissue      ", new ImageIcon("./ico/ico-frame.gif"));
	    addT.addActionListener(new ListAddTissue()); 
	    
	    add = new JPanel();	  
		  add.setBorder(BorderFactory.createTitledBorder(
		  	" Add a new search criteria : ")); 
			add.setForeground(Color.darkGray);
			add.setLayout(new GridBagLayout());
			addComponent(add, addT, 0, 0, 1, 1, GridBagConstraints.CENTER);
			addComponent(add, addD, 0, 1, 1, 1, GridBagConstraints.CENTER); 								
		}
		
		/**
		 * Set the frame in state visible or invisible, 
		 *	and set frame location on screen according to the screen size.
		 * @param b a boolean indicating the frame visibility value
		 */
	 	public void setVisible(boolean b ) {
	    if(b) {
	      Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	      int largeurEcran = screenSize.width;
	      int hauteurEcran = screenSize.height;
	      int largeurFrame = getSize().width;
	      int hauteurFrame = getSize().height;
	      int posX = (largeurEcran-largeurFrame)/2;
	      int posY = (hauteurEcran-hauteurFrame)/2;
	      setLocation(posX, posY);
	    }
	    super.setVisible(b);
	  }
	  
	  /**
		 * Load the frame's auto resizer module.
		 */
		private void loadAutoResizer() {
			this.addComponentListener(new ComponentAdapter() {
	    	public void componentResized(ComponentEvent evt) { 
	      	validate();
	      	repaint();
	    	}
	  	});
		}
	  
	  /**
	   * ReBuild search panels for a loaded search (previously saved).
	   * @param searchParamSet the search parameter set containing 
	   *	previously saved search parameter
	   */
	  private void buildSearchPanels(ClientSearchParameterSet searchParamSet) {
	  	ClientSearchParameter sParam;
	  	ClientSearchPanel ccsp;
	  	for (int i = 0; i < searchParamSet.getNbParameter(); ++i) {	
	  	 	sParam = searchParamSet.getSearchParameter(i);
	  		if (!sParam.getType()) {
	  			ccsp = new ClientSearchPanel(sParam.getType(), 
	  					searchParamSet.getSpecieSearch(), sParam.getNumPanel(), 
	  					crm.toObjectTable(tableEST.HTissue), 
	  					tableEST.HTissue.get(""+sParam.getHValue()), 
	  					sParam.getOperation(), sParam.getValue(), crm);	  			
	  		}
	  		else {
	  			ccsp = new ClientSearchPanel(sParam.getType(), 
	  					searchParamSet.getSpecieSearch(), sParam.getNumPanel(), 
	  					crm.toObjectTable(tableEST.HDevStage),
	  					tableEST.HDevStage.get(""+sParam.getHValue()), 
	  					sParam.getOperation(), sParam.getValue(), crm);
	  		}
	  		ccsp.getRemoveButton().addActionListener(new ListRemove());
				ccsp.getFilterButton().addActionListener(new ListFilter());
				addSearchPanelInList(ccsp);
				addComponent(options, ccsp, 0, i+1, 3, 1, GridBagConstraints.CENTER);	
	  	}
	  }
	  
	  /**
	   * Get the current search panel list.
	   * @return the list of created search panel
	   */		
		public ArrayList getSearchPanelList() {
			return this.panelList;
		}
		
		/**
		 * Remove a search panel from the list 
		 *	when the user has choose to remove a criteria.
		 * @param numPanel the number of the panel to remove
		 */
		public void removeSearchPanelFromList(int numPanel) {
			for (int i = 0; i < panelList.size(); ++i) {
				ClientSearchPanel ccsp = (ClientSearchPanel)panelList.get(i);
				if (ccsp.getNumPanel() == numPanel) {
					panelList.remove(i);
					break;
				}
			} 	
		}
		
		/**
		 * Add a search panel in the list when the user has choose to add a criteria
		 * @param ccsp the search panel to add in the list
		 */
		public void addSearchPanelInList(ClientSearchPanel ccsp) {
			this.panelList.add(ccsp);
		}
		
		/**
		 * Manage simply the layout to add component in the container. 
		 * Create a GridBagConstraints to add simply and more precisely components.
		 *	By default, the component is centered on the destination container. 
		 * @param container the container to update
		 * @param component the component to add
		 * @param x the horizontal position wanted
		 * @param y the vertical position wanted
		 * @param width the number of place to take in width
		 * @param height the number of place to take in height
		 * @param anchor the position of the component
		 */
		private void addWithConstrains(Container container, Component component, 
		    										int x, int y, int width, int height, int anchor) {
		  Insets insets = new Insets (2, 2, 2, 2);
		  GridBagConstraints gbc = new GridBagConstraints();
		    	
		  gbc.gridx = x; gbc.gridy = y;
		  gbc.gridwidth = width; gbc.gridheight = height;
		  gbc.fill = GridBagConstraints.NONE; gbc.anchor = anchor;
		  gbc.weightx = 1.0; gbc.weighty = 1.0;
		  gbc.ipadx = 0; gbc.ipady = 0;
		  gbc.insets = insets;
		
		  LayoutManager lm = container.getLayout();
		  ((GridBagLayout) lm).setConstraints(component, gbc);
		  container.add(component);
		}
		
		/**
		 * Simply add a component to the container passed as argument.
		 *	By default, the component is centered on the destination container. 
		 * @param container the container to update
		 * @param component the component to add
		 * @param x the horizontal position wanted
		 * @param y the vertical position wanted
		 * @param width the number of place to take in width
		 * @param height the number of place to take in height
		 */
		protected void addComponent(Container container, Component component, 
		  											int x, int y, int width, int height) {
		  addWithConstrains(container, component, x, y, width, height, 
																							 GridBagConstraints.NORTHWEST);
		}
		
		/**
		 * Simply add a component to the container passed as argument.
		 * @param container the container to update
		 * @param component the component to add
		 * @param x the horizontal position wanted
		 * @param y the vertical position wanted
		 * @param width the number of place to take in width
		 * @param height the number of place to take in height
		 * @param position the position of the component
		 */ 	
		protected void addComponent(Container container, Component component, 
		  										int x, int y, int width, int height, int position) {
		  addWithConstrains(container, component, x, y, width, height, position);
		}
		
		/**
		 * The ListSpecieBoxChange class is a Listener used to refresh automatically 
		 *	the selected specie informations when the user modify his choice
		 */
		class ListSpecieBoxChange implements ActionListener, Runnable {
	    public void actionPerformed(ActionEvent event) {				
				selectedSpecie = specieBox.getSelectedIndex();
				specieCode = (String)cp.INSTALLED_SPECIES[selectedSpecie][0];	
				new Thread(this).start();								
	    }
	    
	    public void run() {
	    	specieInfoText = crm.get_INSTALLED_SPECIE_INFO(specieCode);
	    	specieInfo.setEditable(true);
				specieInfo.setText(specieInfoText);
				specieInfo.setEditable(false);
	      sp.setVisible(true); 
	      sp.repaint();
	    }
	  }
	  
	  /**
		 * The ListenerExportAssocTXT class is a Listener used to export
		 *	current association between NCBI Unigene data and TIGR TGI Clusters 
		 *	in the database for the selected specie. 
		 *	The export format is *.txt (Text Mode) 
		 */
	  public class ListenerExportAssocTXT implements ActionListener, Runnable {
	    Thread exporter;
	    
	    public void actionPerformed(ActionEvent event) {				
				if (exporter != null && exporter.isAlive())
		  		return;
		  	else {
		  		statusBar.setTemporaryInfo(
						" Exporting NCBI Unigene/TIGR TGI association..."
					, 100000);		
		  		exporter = new Thread(this);
		  		exporter.start();	  		
				}
			}
			
			public void run() {
				try {
					String sp = specieCode;
		  		StringBuffer assoc = crm.checkAssociationInfo(sp);
		  		BufferedWriter out = new BufferedWriter(
		  			new FileWriter(sp+"-assoc.txt"));
		  			
		  		StringTokenizer st = new StringTokenizer(assoc.toString(),"\n");
	      	while(st.hasMoreTokens())
						out.write(st.nextToken() + "\n");
					out.close();
					statusBar.setTemporaryInfo(
						" NCBI Unigene/TIGR TGI association written in "+sp+"-assoc.txt"
					, 3000);
				}
				catch (Exception e) {
					statusBar.setTemporaryInfo(
						" ERROR : in exporting NCBI Unigene/TIGR TGI in "+sp+"-assoc.txt !"
					, 3000);
				}
	  	} 
		}
		
		/**
		 * The ListenerExportAssocCSV class is a Listener used to export
		 *	current association between NCBI Unigene data and TIGR TGI Clusters 
		 *	in the database for the selected specie. 
		 *	The export format is *.csv (Comma Separated Value Mode) 
		 */
		public class ListenerExportAssocCSV implements ActionListener, Runnable {	    
	    Thread exporter;
	    
	    public void actionPerformed(ActionEvent event) {				
				if (exporter != null && exporter.isAlive())
		  		return;
		  	else {
		  		statusBar.setTemporaryInfo(
						" Exporting NCBI Unigene/TIGR TGI association..."
					, 100000);		
		  		exporter = new Thread(this);
		  		exporter.start();	  		
				}
			}
			
			public void run() {
	  		try {
					String sp = specieCode;
		  		StringBuffer assoc = crm.checkAssociationInfo(sp);
		  		
		  		PrintWriter out = new PrintWriter(new FileWriter(sp+"-assoc.csv"));
    			ClientCSVWriter csv = new ClientCSVWriter(out);
					
					StringTokenizer st1 = new StringTokenizer(assoc.toString(),"\n");
	      	while(st1.hasMoreTokens()) {
						StringTokenizer st2 = new StringTokenizer(st1.nextToken(),"\t");
						while(st2.hasMoreTokens()) {
							csv.writeField(st2.nextToken());
							csv.endBlock();
						}
						csv.endLine();						
					}
					csv.close();
					out.close();
					statusBar.setTemporaryInfo(
						" NCBI Unigene/TIGR TGI association written in "+sp+"-assoc.csv"
					, 3000);
				}
				catch (Exception e) {
					statusBar.setTemporaryInfo(
						" ERROR : in exporting NCBI Unigene/TIGR TGI in "+sp+"-assoc.csv !"
					, 3000);
				}
	  	} 
		}
		
		/**
		 * The ListOKs class is a Listener used to accept the specie choice
		 *	and build the search options panel to allow user to choose the
		 *	search parameter he wants.
		 */
	  class ListOKs implements ActionListener, Runnable {
	    public void actionPerformed(ActionEvent event) {
				selectedSpecie = specieBox.getSelectedIndex();
				specieCode = (String)cp.INSTALLED_SPECIES[selectedSpecie][0];
				new Thread(this).start();				
	    }
	    
	    public void run() {
	    	// update the content of HashMaps according to the specie
	    	tableEST.HTissue = crm.get_code_converter_tissue(specieCode);					
	  		tableEST.HDevStage = crm.get_code_converter_dev_stage(specieCode);	

				csp = new ClientSearchParameterSet(specieCode);
	      sp.setVisible(false);      
	      buildSearchOptionsPanel();
	      ClientFrameSearch.this.add(op, BorderLayout.CENTER);
	      setVisible(true); 
	      repaint();
	      
	      tableEST.HTitle = crm.get_code_converter_title(specieCode);	  		
	  		HTissueWage = crm.get_wage_tissue(specieCode);				
				HDevStageWage = crm.get_wage_dev_stage(specieCode);	
	    }
	  }
	  
	  /**
		 * The ListOKo class is a Listener used to accept the list
		 *	of search parameters which was choosen by the user. 
		 * A detection of bad parameter is executed in order to invite the user 
		 *	to correct one or more bad parameters.
		 * When all the parameters are ok, the sorted search paramter set 
		 *	is generated and the search process is run. 
		 */
	  class ListOKo implements ActionListener, Runnable {
	    public void actionPerformed(ActionEvent event) {
	    	try {	   
	    		boolean type;
	    		Character op;
	    		int sel, value, numPanel, wage;
	    		ClientSearchPanel ccsp;
	    		
	    		// Check if parameters are ok
	    		boolean runSearch = this.checkParameters();
	    	 	
	    	 	if (runSearch) {
	    	 		sortByWages();
							
						// Test if a restart point will be used
						int[] searchVector = testRestartPoints();

						tableCluster.setClusterSet(new ClientClusterSet());
						tableCluster.setSavable(false);
						tableEST.setESTSet(new ClientESTSet(0));
	  				tableGO.setGOSet(new ClientGOSet(0));
						
						//save the search parameter set for future search modification
						lastSearch = csp;
						
						if (csp.getNbParameter() > 0) {
							tabbedPane.setEnabledAt(1, false);
							tabbedPane.setSelectedIndex(0);	
							statusBar.setTemporaryInfo(
		    				" Please wait while computing your search...", 10000000);	
		    			
		    			// run the main search thread																			
							cts = new ClientThreadSearch(
								cp, csp, new ClientRequestSearch(con, cp), 
								searchVector, statusBar.getIP().replace('.', '_'));	
			    		cts.start(); 	
			    		new Thread(this).start();
		    		}
		    		else {
		    			statusBar.setTemporaryInfo(
		    				" ERROR : Your search is empty of parameters...", 3000);	
		    			return;
		    		}
	    	 	}	
				}
				catch (Exception e) {
					PrintException(e);
				}
			}
			
			public void run() {	  					
	  		try {	
	  			cts.join();
	  			tableCluster.setClusterSet(cts.getClusterSet());
					long time = cts.getSearchTime();
					int nbCluster = tableCluster.getClusterSet().getNbCluster();					
					
					String specieCode = tableCluster.getClusterSet().getSpecie();
					tableCluster.getClusterSet().getSearchParameterSet().setNCBIVersion(
	  				cp.getNCBIVersion(specieCode));
	  			tableCluster.getClusterSet().getSearchParameterSet().setTIGRVersion(
	  				cp.getTIGRVersion(specieCode));
					
					if (nbCluster == 0) {
						statusBar.setTemporaryInfo(
		  				" bioECS Search Done : No Result found in "+time+" second(s) !"
		  			, 3000);  				
					}
					else {
						statusBar.setTemporaryInfo(
		  				" bioECS Search Done : "+nbCluster+" clusters found in "+time+" second(s) !"
		  			, 3000);  
		  			tableCluster.setSavable(true);						
					}
					tabbedPane.setEnabledAt(1, true);	
					stoppedSearch = false;	  			
		  		repaint();		 
	  		}	
	  		catch (Exception e) {
	  			PrintException(e);
	  		}
			}
				
			/**
			 * Check if parameters are valids.
			 * Validity criteria :
			 *	1) VALID :  value > 0%   and   value <= 100%
			 *  2) NOT VALID : value =< 0%   and   value > 100% 
			 *  3) NOT VALID : sum value for tissue > 100%
			 *  4) NOT VALID : sum value for devStage > 100%
			 *	5) NOT VALID : all duplicated parameters
			 */
			private boolean checkParameters() {
		    boolean runSearch = true;
		    int totalTissue = 0, totalDevStage = 0;
		    boolean flagOpTissue = false, flagOpDevStage = false;
		    ClientSearchPanel ccsp;
		    ClientSearchPanel ccspTmp;
		    
		    for (int i = 0; i < panelList.size(); ++i) {
		    	ccsp = (ClientSearchPanel)panelList.get(i);		    		
		    	
		    	boolean type = ccsp.getTypeSearch();
		    	Character op = ccsp.getOperation();
		    	int sel = ccsp.getDataBoxSelectedCode();
		    	
		    	try {
		    		int value = ccsp.getValue();
		    		//Test for invalid specified percent value
		    		if (value >= 0 && value <= 100) {
		    			if (ccsp.getOperation().equals(new Character('<')) 
		    					&& value == 0) {
		    				ccsp.setErrorPercentText();
		    				runSearch = false;
		    			}
		    			else if (ccsp.getOperation().equals(new Character('>')) 
		    					&& value == 100) {
		    				ccsp.setErrorPercentText();
		    				runSearch = false;
		    			}
		    			else if (ccsp.getOperation().equals(new Character('=')) 
		    					&& value == 0) {
		    				ccsp.setErrorPercentText();
		    				runSearch = false;
		    			}
		    			else {
		    				//Test for avoid asking more than 100% of a parameter type.
		    				if (ccsp.getOperation().equals(new Character('>'))) {
		    					if (!ccsp.getTypeSearch()) {
		    						totalTissue += value;
		    						flagOpTissue = true;
		    						if (totalTissue >= 100) {
		    							ccsp.setErrorMaximumText();
		    							runSearch = false;
		    						}
		    						else ccsp.setNoErrorPercentText();	 	    						
		    					}
		    					else if (ccsp.getTypeSearch()) {
		    						totalDevStage += value;
		    						flagOpDevStage = true;
		    						if (totalDevStage >= 100) {
			    						ccsp.setErrorMaximumText();
			    						runSearch = false;
		    						}
		    						else ccsp.setNoErrorPercentText();	
		    					}
		    				}
		    				else if (ccsp.getOperation().equals(new Character('='))) {
									if (!ccsp.getTypeSearch()) {
		    						totalTissue += value;
		    						if (flagOpTissue && totalTissue >= 100) {
		    							ccsp.setErrorMaximumText();
		    							runSearch = false;
		    						}
		    						else if (!flagOpTissue && totalTissue > 100) {
		    							ccsp.setErrorMaximumText();
		    							runSearch = false;
		    						}
		    						else ccsp.setNoErrorPercentText();	 	    						
		    					}
		    					else if (ccsp.getTypeSearch()) {
		    						totalDevStage += value;
		    						if (flagOpDevStage && totalDevStage >= 100) {
			    						ccsp.setErrorMaximumText();
			    						runSearch = false;
		    						}
		    						else if (!flagOpDevStage && totalDevStage > 100) {
		    							ccsp.setErrorMaximumText();
		    							runSearch = false;
		    						}
		    						else ccsp.setNoErrorPercentText();	
		    					} 					
		    				}    				
		    				else ccsp.setNoErrorPercentText();
		    			}
		    		}
		    		else {
		    			ccsp.setErrorPercentText();
		    			runSearch = false;
		    		}
		    	}
		    	catch (NumberFormatException nfe) {
						ccsp.setErrorPercentText();
						runSearch = false;
					}
					
					//Test for duplicated parameter
		    	for (int j = i+1; j < panelList.size(); ++j) {
		    		ccspTmp = (ClientSearchPanel)panelList.get(j);
		    		if (type == ccspTmp.getTypeSearch() 
		    					&& sel == ccspTmp.getDataBoxSelectedCode()) {
		    			ccsp.setErrorDuplicateText();
		    			runSearch = false;
		    			break;
		    		}		    		
		    	}					
				}
				if (!runSearch) {
					statusBar.setTemporaryInfo(
						" Please correct marked invalid parameter(s) in your search !"
					, 3000);
				}
				return runSearch;
			}
			
			/**
	 		 * Sort the set of parameters by wages associated with each parameter.
	 		 */
			private void sortByWages() {
				boolean type;
	    	Character op;
	    	int sel, value, numPanel, wage;
	    	ClientSearchPanel ccsp;
	    		
				csp = new ClientSearchParameterSet(specieCode);
  	 		for (int i = 0; i < panelList.size(); ++i) {
	    		ccsp = (ClientSearchPanel)panelList.get(i);
	    		type = ccsp.getTypeSearch();
	    		op = ccsp.getOperation();
	    		sel = ccsp.getDataBoxSelectedCode();
	    		value = ccsp.getValue();
	    		numPanel = ccsp.getNumPanel();		    		
	    		if (!type) wage = Integer.parseInt(
	    			(String)HTissueWage.get(new String(""+sel)));
					else wage = Integer.parseInt(
						(String)HDevStageWage.get(new String(""+sel)));
					
					csp.addNewSearchParameter(
						new ClientSearchParameter(numPanel, wage, type, sel, op, value));
				}				
			}
			
			/**
	 		 * Compare the last ClientParameterSet with the new one to decide 
	 		 *	if the new search is more or less restrictive than the last one.
	 		 */
			private int[] testRestartPoints() {
				// 0: restart at previous result, compute result with operation & value
				// 1: no match... normal processing				
				int[] restartStatus = new int[csp.getNbParameter()];
				
				for (int i = 0; i < restartStatus.length; ++i)
					restartStatus[i] = 1;
								 
				if (lastSearch == null || stoppedSearch
						|| lastSearch.getSpecieSearch() != csp.getSpecieSearch())
					return restartStatus;	
	
				ClientSearchParameter cspLast;
				ClientSearchParameter cspCurrent;								
				for (int i = 0; i < lastSearch.getNbParameter(); ++i) {
					cspLast = lastSearch.getSearchParameter(i);
					cspCurrent = csp.getSearchParameter(i);
									
					if (cspCurrent == null)
						return restartStatus;											
					else {							
						if ((!cspLast.getType() && !cspCurrent.getType())
								|| (cspLast.getType() && cspCurrent.getType())) {							
							if (cspLast.getHValue() == cspCurrent.getHValue()) {									
								restartStatus[i] = 0;								
							}
							else return restartStatus;
						}
						else return restartStatus;
					}						
				}			
				return restartStatus;
			}
	 	}

	  /**
	   * The ListCancelo listener is used to show the specie selection panel.
	   */
	  class ListCancelo implements ActionListener, Runnable {
	    public void actionPerformed(ActionEvent event) {
	      panelList.clear();
	      
	      tableEST.setESTSet(new ClientESTSet(0));
	  		tableGO.setGOSet(new ClientGOSet(0));
				
				op.setVisible(false);
	      remove(sp);
	      new Thread(this).start();
	    }
	    
	    public void run() {
	      specieInfoText = crm.get_INSTALLED_SPECIE_INFO(specieCode);
				specieInfo.setText(specieInfoText);	      
	      add(sp, BorderLayout.CENTER);
	      sp.setVisible(true);      
	      repaint();
	    }
	  }
	  
	  class ListAddTissue implements ActionListener {
	    public void actionPerformed(ActionEvent event) {	      
	      op.setVisible(false);      
				Object[][] toSet = crm.toObjectTable(tableEST.HTissue);
				
				if (toSet != null) {
					numCriteria++;
					ClientSearchPanel csp = new ClientSearchPanel(
						false, specieCode, numCriteria, toSet, crm);
					csp.getRemoveButton().addActionListener(new ListRemove());
					csp.getFilterButton().addActionListener(new ListFilter());
					addSearchPanelInList(csp);
					addComponent(
						options, csp, 0, numCriteria+1, 3, 1, GridBagConstraints.CENTER);
				}				
	      op.setVisible(true);   	 		  
	      repaint();         
	    }
	  }
	  
	  class ListAddDevStage implements ActionListener {
	    public void actionPerformed(ActionEvent event) {
	    	op.setVisible(false);      
				Object[][] toSet = crm.toObjectTable(tableEST.HDevStage);
				
				if (toSet != null) {									
					numCriteria++;			
					ClientSearchPanel csp = new ClientSearchPanel(
						true, specieCode, numCriteria, toSet, crm);
					csp.getRemoveButton().addActionListener(new ListRemove());
					csp.getFilterButton().addActionListener(new ListFilter());
					addSearchPanelInList(csp);
					addComponent(
						options, csp, 0, numCriteria+1, 3, 1, GridBagConstraints.CENTER);	
				}			
	      op.setVisible(true);   	 		  
	      repaint();    
	    }
	  }
	    		
		class ListFilter implements ActionListener, Runnable {
			ClientSearchPanel csptmp;
	    public void actionPerformed(ActionEvent event) {
	      JButton cp = (JButton)event.getSource();
	      csptmp = (ClientSearchPanel)cp.getParent();	    	
	    	new Thread(this).start();
	    }
	    
	    public void run() {
	    	try {
		    	HashMap tmp = new HashMap();
		    	    	
		    	if (!csptmp.getTypeSearch())
		    		tmp = crm.get_code_converter_tissue(
		    			specieCode, csptmp.getEnteredPattern());
		    	else if (csptmp.getTypeSearch())
		    		tmp = crm.get_code_converter_dev_stage(
		    			specieCode, csptmp.getEnteredPattern());	 
		    	
		    	Object[][] toSet = crm.toObjectTable(tmp);     
		      csptmp.setData(toSet);	          
		      csptmp.remove(csptmp.getDataBox());
		      csptmp.setDataBox(new JComboBox(toSet[1]));
		      addComponent(
		      	csptmp, csptmp.getDataBox(), 0, 1, 3, 1, GridBagConstraints.CENTER);		      
		      csptmp.setVisible(true);
		      csptmp.repaint();
	      }	
	  		catch (Exception e) {
	  			PrintException(e);
	  		}    	
	    }
	  }
	  
	  class ListRemove implements ActionListener {
		  public void actionPerformed(ActionEvent event) {
		   	JButton cp = (JButton)event.getSource();
	      ClientSearchPanel csp = (ClientSearchPanel)cp.getParent();
	      removeSearchPanelFromList(csp.getNumPanel());		   
		    csp.setVisible(false); 
		    repaint();	      
		  }
		} 
	}
	

	public class BioUpdater implements Runnable {
    public BioUpdater(ClientClusterSet ccset) {
    	String codeList = "( ";
			for (int i = 0; i < ccset.getNbCluster(); ++i) {
				codeList += "`old_code_clu` = "+ccset.getCluster(i).getClusterCode()+" OR ";
			}
	  	codeList = codeList.substring(0, codeList.length()-3) + ")";
	  			
			int nbParam = ccset.getSearchParameterSet().getNbParameter();
			int[] vector = new int[nbParam];
			for (int i = 0; i < nbParam; ++i)
				vector[i] = 1;
			
			statusBar.setTemporaryInfo(
	  		" Please wait while updating data...", 5000);

	  	cts = new ClientThreadSearch(
	  		cp, ccset.getSearchParameterSet(), 
	  		new ClientRequestSearch(con, cp), vector, clientIP);
	  	cts.setOldClusterCodes(codeList);	  	
	  	cts.start();
      new Thread(this).start();
    }
    
    public void run() {
    	try {	
				cts.join();
				tableCluster.setClusterSet(cts.getClusterSet());
				long time = cts.getSearchTime();					

				statusBar.setTemporaryInfo(
	  			" bioECS data update done in "+time+" second(s) !"
	  		, 3000); 
	  		String specieCode = tableCluster.getClusterSet().getSpecie(); 
	  		tableCluster.getClusterSet().getSearchParameterSet().setNCBIVersion(
	  			cp.getNCBIVersion(specieCode));
	  		tableCluster.getClusterSet().getSearchParameterSet().setTIGRVersion(
	  			cp.getTIGRVersion(specieCode));
	  			
	  		tableCluster.setSavable(true);							  		
				stoppedSearch = false;	  			
	  		repaint();		 
			}	
			catch (Exception e) {
				PrintException(e);
			}
    }
  }
  
  
  /* Classe interne de paramtrage du fileFilter */
  public class BioFilter extends FileFilter {
    String desc = ".clu";
    private ArrayList extensions;

    public BioFilter() {
      super();
      extensions = new ArrayList();
      extensions.add(".clu");
    }

    public boolean accept(File f) {
      if (f.isDirectory()) 
      	return true;
      String name = f.getName().toLowerCase();
      if (name.endsWith(".clu")) return true;
      return false;
    }

    public String getDescription() {
      return desc;
    }
  }
	
	/**
	 * The BioLabel class extends JLabel. 
	 *	A BioLabel object is a simple JLabel with a mono-spaced font which allows 
	 *	a determinitic lenght (in character) for the label.
	 *	Then a BioLabel is designed with an etched border.
	 */
	private class BioLabel extends JLabel {
		private int lenght;
		
		/**
		 * Construct a new BioLabel object.
		 * @lg the lenght to set (in character)
		 */
		public BioLabel(int lg) {
			super();
			this.lenght = lg;	
		}
		
		/**
		 * Construct a new BioLabel object by specifiyng the text to show.
		 * @str the text to show on the label
		 * @lg the lenght to set (in character)
		 */
		public BioLabel(String str, int lg) {
			super();
			this.lenght = lg;	
			this.setFont(new Font("MONOSPACED",Font.BOLD,11));
	    this.setBorder(BorderFactory.createEtchedBorder());	
			this.setLabelText(str);
		}
		
		/**
		 * Set the text to show on the label by filling empty zone with spaces.
		 *	NOTE: If the text to show is too long, the end is replaced by "..."
		 * @str the text to show on the label
		 */
		public void setLabelText(String str) {
			int emptySpace = lenght - str.length();
			String toShow = new String();			
			if (emptySpace < 0)
				toShow = str.substring(0,str.length()-3) + "...";
			else toShow = setSpace(str, emptySpace);	 
			this.setText(toShow);	
		}
		
		/**
		 * Return a String of space to complete the label lenght
		 * @str the text to show on the label
		 * @nbSpace the number of space to set
		 * @return a string of space characters
		 */
		private String setSpace(String str, int nbSpace) {
			String ret = new String(str);
			for (int i = 0; i < nbSpace; ++i)
				ret += " ";
			return ret;	
		}	
	} 
	
 	/**
   * The ClientStatusBar class implements the status bar
   *	which is show at bottom of the ClientGUI frame.
   */
	private class ClientStatusBar extends JPanel {
		private JLabel statusInfo;
		private BioSystemInfo sysInfo;
		private Timer timer;
		
		/**
		 * Create a new ServerStatusBar object.
		 */
		public ClientStatusBar() {
			super();					
			this.setBorder(BorderFactory.createEtchedBorder());
		  this.statusInfo = new JLabel(
		  	" ", new ImageIcon("./ico/ico-inform.gif"), 0);
		  this.statusInfo.setFont(new Font("MONOSPACED", Font.PLAIN, 12));    
		  this.sysInfo = new BioSystemInfo();
		       
		  this.setLayout(new GridBagLayout());
		  addComponent(this, statusInfo, 0, 0, 1, 1);   			
			addComponent(this, sysInfo, 0, 1, 1, 1);	
		}
		
		/**
		 * Set a status information text (on the top label of the status bar).
		 * @param str the text to set
		 */
		public void setStatusInfo(String str) {
		 	statusInfo.setText(str);
		}
		
		/**
		 * Return the client current IP address which is show in the status bar.
		 * @return a string object containing the IP address
		 */
		public String getIP() {
			return this.sysInfo.ip;
		}
		
		/**
		 * Manage simply the layout to add component in the container. 
		 * Create a GridBagConstraints to add simply and more precisely components.
		 *	By default, the component is centered on the destination container. 
		 * @param container the container to update
		 * @param component the component to add
		 * @param x the horizontal position wanted
		 * @param y the vertical position wanted
		 * @param width the number of place to take in width
		 * @param height the number of place to take in height
		 */ 
		private void addWithConstrains(Container container, Component component, 
		  							int x, int y, int width, int height) {
		  Insets insets = new Insets (2, 2, 2, 2);
		  GridBagConstraints gbc = new GridBagConstraints();
		  	
		  gbc.gridx = x; gbc.gridy = y;
		  gbc.gridwidth = width; gbc.gridheight = height;
		  gbc.fill = GridBagConstraints.NONE; 
		  gbc.anchor = GridBagConstraints.NORTHWEST;
		  gbc.weightx = 1.0; gbc.weighty = 1.0;
		  gbc.ipadx = 0; gbc.ipady = 0;
		  gbc.insets = insets;
		
		  LayoutManager lm = container.getLayout();
		  ((GridBagLayout) lm).setConstraints(component, gbc);
		  container.add(component);
		}
		
		/**
		 * Simply add a component to the container passed as argument.
		 *	By default, the component is centered on the destination container. 
		 * @param container the container to update
		 * @param component the component to add
		 * @param x the horizontal position wanted
		 * @param y the vertical position wanted
		 * @param width the number of place to take in width
		 * @param height the number of place to take in height
		 */
		private void addComponent(Container container, Component component, 
															int x, int y, int width, int height) {
		  addWithConstrains(container, component, x, y, width, height);
		}
		
		/**
		 * Set a temporary information message on the top label of the status bar.
		 * @param text the information text to show
		 * @ms the show time for the information text (in milliseconds)
		 * NOTE: when the time is out, the message : Ready !    is show
		 */
		public void setTemporaryInfo(String text, long ms) {
				setStatusInfo(text);
		  if(timer != null)
		  	timer.cancel();
		  timer = new Timer();
		
		  timer.schedule(new TimerTask() {
		   	public void run() {
		      setStatusInfo(" Ready ! ");
		    }
		  },ms);
		}

		/**
	   * The BioSystemInfo class allows to retrieve informations
	   *	about the system running this program, the user logged in, ...
	   */
		private class BioSystemInfo extends JPanel implements Runnable {
			public String ip;
			
			private BioLabel systemInfo;
		  private BioLabel date; 
		  private BioLabel time;
		  private Thread current;
		  private int numDay;
		  
		  /**
		   * Construct a new BioSystemInfo object.
		   */  
		  public BioSystemInfo() {
		    super();
		    this.setBorder(BorderFactory.createEtchedBorder());       		
		    time       = new BioLabel(20);       
		    date       = new BioLabel(40);      
		    systemInfo = new BioLabel(100);
	      
		    this.getDayDate();
		    this.getSystemInfo();
		        
		    this.setLayout(new GridBagLayout());		
				addComponent(this, time, 0, 0, 1, 1);
				addComponent(this, date, 1, 0, 1, 1);
				addComponent(this, systemInfo, 2, 0, 1, 1);      
		        
		    current = new Thread(this);
		    current.start();               
		    setVisible(true);
			}
			
			/**
			 * Get choosen system parameters and show them in the status bar.
			 */
			public void getSystemInfo() {   
		    String str = "";
		    Properties systemProperties = System.getProperties();
		    Enumeration enum = systemProperties.propertyNames();
		    while (enum.hasMoreElements()) {
		      String key = (String)enum.nextElement();
		      if (key.equals("java.vm.version"))
		      	str += " JDK v" + systemProperties.getProperty(key);
		      else if (key.equals("os.arch"))
						str += "     running on " + systemProperties.getProperty(key);
				 	else if (key.equals("user.name"))
						str += "     by " + systemProperties.getProperty(key);	
		    }
		    try {
		    	ip = InetAddress.getLocalHost().getHostAddress();
		    	str += "             Your IP : "+ip;
		    }
		    catch (Exception e) { }	  
		    systemInfo.setLabelText(str);             
			}
		
			/**
			 * This method is used to get the current time.
			 */
		  public void run() {
		    Thread tmp = Thread.currentThread() ;	            
		    while(tmp == current) {
		      hourInDay() ;
		    	try { Thread.sleep(1000); }
		    	catch(InterruptedException e) { }
		    }
		  }
			
			/**
			 * Get the current date in a parsed string.
			 */
		  private void hourInDay() {
		    int hour, minut, second;
		    Calendar cal = Calendar.getInstance();
		    String str = "";
		        
		    hour = cal.get(Calendar.HOUR_OF_DAY);         
		    if (hour < 10)	str += "0" + Integer.toString(hour)+" : ";
		    else	str += Integer.toString(hour) + " : ";
		                
		    minut = cal.get(Calendar.MINUTE);
		    if(minut < 10)	str += "0" + Integer.toString(minut)+" : ";
		    else str += Integer.toString(minut) + " : " ;
		   
		   	second = cal.get(Calendar.SECOND);
		    if(second < 10) str += "0" + Integer.toString(second);
		    else str += Integer.toString(second) ;
		                            
		    time.setLabelText(str);
		    int newDay = cal.get(Calendar.DAY_OF_WEEK);           
		    if(newDay != numDay) {
		       getDayDate();
		    }           
			}
			
			/**
			 * Manage the change of day which occurs at midnight.
			 */
		  private void getDayDate() {
		    int dWeek, dMonth, month, year;
		    String str = "";
		            
		    Calendar now = Calendar.getInstance();        
		    dWeek = now.get(Calendar.DAY_OF_WEEK);
		    dMonth = now.get(Calendar.DAY_OF_MONTH);
		    numDay = now.get(Calendar.DAY_OF_WEEK);            
		    month = now.get(Calendar.MONTH);
		    year = now.get(Calendar.YEAR);
		            
		    str += dayWeek(dWeek);
		    str += ", "+ Integer.toString(dMonth) + " " ;	            
		    str += monthYear(month);
		    str += " "+ Integer.toString(year);
		    date.setLabelText(str);
		  }
		  
		  /**
		   * Get a string at place of the day number.
		   * @param the day number in week
		   * @return a string containing the day name
		   */  
		  private String dayWeek(int j) {
		   	switch(j) {
		      case 1 : return "Sunday" ;
		      case 2 : return "Monday" ;
		      case 3 : return "Tuesday" ;
		      case 4 : return "Wednesday" ;
		      case 5 : return "Thursday" ;
		      case 6 : return "Friday" ;
		      case 7 : return "Sunday" ;
		    }
		    return "Invalid Day";
		  }
			
			/**
		   * Get a string at place of the month number.
		   * @param the month number in year
		   * @return a string containing the month name
		   */ 
		  private String monthYear(int j) {
		   	switch(j) {
		      case 0 : return "January" ;
		      case 1 : return "February" ;
		      case 2 : return "March" ;
		      case 3 : return "April" ;
		      case 4 : return "May" ;
		      case 5 : return "June" ;
		      case 6 : return "July" ;
		      case 7 : return "August" ;
		      case 8 : return "September" ;
		      case 9 : return "October" ;
		      case 10 : return "November" ;
		      case 11 : return "December" ;
				}
				return "Invalid Month";
			}     
		}	
	}
}	  

