package client;

import java.io.*;
import java.util.*;

/**
 * This class stores current parameters for the client program.
 */
public class ClientParameters implements Serializable {		
	public String INSTALLED_SPECIES[][];
	public boolean PRINT_DEBUG_INFO = false;
	public String LOGIN = "";
	public String SERVER = "";
	public String PORT = "3306";
	
	public HashMap TC_REPORT_MAP = new HashMap();
	
	public String CONNECTION_ERROR = 
		"ERROR : Can't connect to specified database\n\n" +
  	"  Possible Source of this problem :\n" +
  	"   - Please check your database connection parameters\n" +
  	"   - Please check your network connection parameters\n" +
  	"   - MySQL Server may be down";
	
	public ClientParameters() {
		buildTCReportMap();		
	}
	
	private void buildTCReportMap() {
		TC_REPORT_MAP.put("Hs", "human");
		TC_REPORT_MAP.put("Mm", "mouse");
		TC_REPORT_MAP.put("Rn", "rat");
		TC_REPORT_MAP.put("Bt", "cattle");
		TC_REPORT_MAP.put("Ssc", "pig");
		TC_REPORT_MAP.put("Cfa", "dog");
		TC_REPORT_MAP.put("Dm", "drosoph");
		TC_REPORT_MAP.put("Dr", "zfish");
		TC_REPORT_MAP.put("Xl", "xenopus");
		TC_REPORT_MAP.put("Str", "x_tropicalis");
		TC_REPORT_MAP.put("Ola", "o_latipes");
		TC_REPORT_MAP.put("Gga", "g_gallus");
		TC_REPORT_MAP.put("Cin", "c_intestinalis");
		TC_REPORT_MAP.put("Ssa", "salmon");
		TC_REPORT_MAP.put("Cel", "elegans");
		TC_REPORT_MAP.put("Sma", "s_mansoni");
		TC_REPORT_MAP.put("Aga", "mosquito");
		TC_REPORT_MAP.put("Ame", "honeybee");
	}
	
	public boolean isInstalled(String specieCode) {
		if (INSTALLED_SPECIES != null) {
			for (int i = 0; i < INSTALLED_SPECIES.length; ++i) {
				if (INSTALLED_SPECIES[i][0].equals(specieCode))
					return true;				
			}			
		}
		return false;	
	}
	
	public int getNCBIVersion(String specieCode) {
		if (INSTALLED_SPECIES != null) {
			for (int i = 0; i < INSTALLED_SPECIES.length; ++i) {
				if (INSTALLED_SPECIES[i][0].equals(specieCode))
					return Integer.parseInt(INSTALLED_SPECIES[i][1]);				
			}			
		}
		return -1;	
	}
	
	public int getTIGRVersion(String specieCode) {
		if (INSTALLED_SPECIES != null) {
			for (int i = 0; i < INSTALLED_SPECIES.length; ++i) {
				if (INSTALLED_SPECIES[i][0].equals(specieCode))
					return Integer.parseInt(INSTALLED_SPECIES[i][4]);				
			}			
		}
		return -1;	
	}
	
	/**
	 * Serialize in a file a ClientParameters object.
	 * @param cp the object to serialize
	 * @param dir the output location directory 
	 */
 	public static void saveClientParameters(ClientParameters cp, String dir) {
	  try {
	    FileOutputStream fos = new FileOutputStream(dir);
	    ObjectOutputStream oos = new ObjectOutputStream(fos);
	    oos.writeObject(cp);
	    oos.close();    
	  }
	  catch(IOException e) { }
 	}
	
	/**
	 * Retrieve a serialized ClientParameters object from a file.
	 * @param dir the file containing the object
	 * @return the ClientParameters object extracted or null if exception 
	 */
  public static ClientParameters getClientParameters(String dir) {
	  try {
	    FileInputStream fis = new FileInputStream(dir);
	    ObjectInputStream ois = new ObjectInputStream(fis);
	    ClientParameters ret = (ClientParameters)ois.readObject();
	    ois.close();
	    return ret;
	  }
	  catch(IOException e) { 
	  	return new ClientParameters();
	  }
	  catch(ClassNotFoundException e) {	
	  	return new ClientParameters();
	  }
	  catch(NullPointerException e) {
	  	return new ClientParameters();
	  }
	}
}