package client;

import java.sql.*;

/**
 * The ClientRequestEST class is used to retrieve EST informations
 *	as the result of a click on a cluster.
 * So, this class is used to manage the 'EST retrieval info' process.
 */
public class ClientRequestEST extends ClientRequest {
	
	/**
   * Construct a new ClientRequestEST object. 
   * @param con the established connection
   * @param cp the current parameters for the program
   */
	public ClientRequestEST(Connection con, ClientParameters cp) {
		super(con, cp);
	}

	/**
	 * Check EST informations about a cluster when click on the cluster.
	 * @param specieCode the specie code
	 * @param cluster the clicked cluster
	 * @return a ClientESTSet object containing the EST informations 
	 *	(return null if problem)
	 */
	public ClientESTSet checkESTInfo(String specieCode, int cluster) {		
		try {
			doRequestSEL(
				"SELECT `code_est`, `title`, `tissue`, `devstage` "+
				" FROM `"+specieCode+"_est`, `"+specieCode+"_lib` "+
				" WHERE ('"+cluster+"' = `code_clu` "+
				" AND `code_lib_in` = `code_lib`);");		
				 
			ClientESTSet ret = new ClientESTSet(cluster); 
			int title = -1, tissue = -1, devstage = -1;
			String est = "";

			boolean run = results.next(); 
			while(run) { 
		    est = results.getString(1);
				title = results.getInt(2);
				tissue = results.getInt(3);
				devstage = results.getInt(4);

				ret.addEST(new ClientEST(est, title, tissue, devstage));									      
		    run = results.next();	
		  }
		  return ret;	  	
    }
    catch (SQLException e) { 
			PrintSQLError(e); 
			return null;		
	 	}	
	}
}

