package client;

import java.sql.*;

/**
 * The ClientRequestGO class is used to retrieve GO informations
 *	as the result of a click on a cluster.
 * So, this class is used to manage the 'GO retrieval info' process.
 */
public class ClientRequestGO extends ClientRequest {
	
	/**
   * Construct a new ClientRequestGO object. 
   * @param con the established connection
   * @param cp the current parameters for the program
   */
	public ClientRequestGO(Connection con, ClientParameters cp) {
		super(con, cp);
	}
	
	/**
	 * Check Go-Functions informations about a cluster when click on the cluster.
	 * @param specieCode the specie code
	 * @param cluster the clicked cluster
	 * @return a ClientGOSet object containing the Go-Functions informations
	 *	(return null if problem)
	 */
	public ClientGOSet checkGOInfo(String specieCode, int clusterCode) {	
		try {
			doRequestSEL(
				"SELECT DISTINCT `code_go`, `term_go_cc`, `type_go_cc`, `gopep` "+
				" FROM `"+specieCode+"_go`, `go_functions` "+
				" WHERE (`code_clu` = '"+clusterCode+"' AND `code_go` = `code_go_cc` "+
				"  AND `gopep` = `gopep_cc`);");	
				
			ClientGOSet ret = new ClientGOSet(clusterCode); 
			String namego = "", chargo = "", genego = "";
			int codego = -1;

			boolean run = results.next(); 
			while(run) { 
		    codego = results.getInt(1);
				namego = results.getString(2);
				chargo = results.getString(3);
				genego = results.getString(4);

				ret.addGOFunction(new ClientGO(codego, namego, chargo.charAt(0), genego));									      
		    run = results.next();	
		  }
		  return ret;	  	
		}
		catch (SQLException e) { 
			PrintSQLError(e); 
			return null;
	  }
	}
}

