package client;

import java.lang.Integer;
import java.awt.event.*;
import javax.swing.event.*; 
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.*;
import java.awt.*;

/**
 * A ClientSearchPanel is an extended JPanel.
 *	This class represents the panel show 
 *	when the user add a new parameter to his search.
 */
public class ClientSearchPanel extends JPanel {
	private Character[] OP = { 
			new Character('='), new Character('<'), new Character('>') };
	private JButton remove;
	private JLabel pattern;
	private JTextField patternT;
	private JButton filter;
	private JComboBox dataBox;
	private JComboBox op;
	private JTextField valueT;
	private JLabel pc;
	
	private Object[][] toShow;
	private int numPanel;
	private boolean type;
	private String specieCode;
	private ClientRequestManager crm;
	
	/**
	 * Construct a new ClientSearchPanel object.
	 * @param type the type oof the parameter (0=tissue, 1=devStage)
	 * @param specieCode the specie selected
	 * @param numPanel the unique panel number
	 * @param toShow the items to show in the data ComboBox
	 * @param crm the request manager which can execute request on the data base
	 */		
	public ClientSearchPanel(boolean type, String specieCode, int numPanel, 
																Object[][] toShow, ClientRequestManager crm) {		
		this.type = type;
		this.specieCode = specieCode;
		this.numPanel = numPanel;				
		this.crm = crm;
		this.toShow = toShow;
		
		remove = new JButton(
			"Remove Criteria" , new ImageIcon("./ico/ico-deleteframe.gif"));			    
		pattern = new JLabel("Pattern Filter : "); 
		patternT = new JTextField(16);  
		filter = new JButton(
			"Filter !", new ImageIcon("./ico/ico-piegraph.gif"));		
			            
		dataBox = new JComboBox(toShow[1]);
		op = new JComboBox(OP);
		valueT = new JTextField("100", 3);  
		pc = new JLabel("%"); 
		
		this.buildPanel();
		this.setVisible(true);	
	}
	
	/**
	 * Construct a new ClientSearchPanel object.
	 * @param type the type oof the parameter (0=tissue, 1=devStage)
	 * @param specieCode the specie selected
	 * @param numPanel the unique panel number
	 * @param toShow the items to show in the data ComboBox
	 * @param selParam the selected item in the data ComboBox
	 * @param selOp the selected operation in the operation ComboBox
	 * @param selValue the selected percent value
	 * @param crm the request manager which can execute request on the data base
	 */	
	public ClientSearchPanel(boolean type, String specieCode, int numPanel, 
														Object[][] toShow, Object selParam, Object selOp, 
														int selValue, ClientRequestManager crm) {	
		this.type = type;
		this.specieCode = specieCode;
		this.numPanel = numPanel;				
		this.crm = crm;
		this.toShow = toShow;
		
		remove = new JButton(
			"Remove Criteria" , new ImageIcon("./ico/ico-deleteframe.gif"));			    
		pattern = new JLabel("Pattern Filter : "); 
		patternT = new JTextField(16);  
		filter = new JButton(
			"Filter !", new ImageIcon("./ico/ico-piegraph.gif"));		
			            
		dataBox = new JComboBox(toShow[1]);
		dataBox.setSelectedItem(selParam);
		op = new JComboBox(OP);
		op.setSelectedItem(selOp);
		valueT = new JTextField(""+selValue, 3);  
		pc = new JLabel("%"); 
		
		this.buildPanel();	  	
		this.setVisible(true);	
	}
	
	/**
	 * Add all the components in the main container of the panel.
	 */
	private void buildPanel() {
		if (!type) setBorder(
			BorderFactory.createTitledBorder(" Tissue Parameter Box :"));
		else setBorder(
			BorderFactory.createTitledBorder(" Development Stage Parameter Box :"));
		this.setForeground(Color.darkGray);		
		this.setLayout(new GridBagLayout());
		addComponent(this, pattern, 0, 0, 1, 1, GridBagConstraints.EAST);
		addComponent(this, patternT, 1, 0, 1, 1, GridBagConstraints.WEST);	
		addComponent(this, filter, 2, 0, 1, 1, GridBagConstraints.WEST);
		addComponent(this, remove, 3, 0, 3, 1, GridBagConstraints.EAST); 
		addComponent(this, dataBox, 0, 1, 3, 1, GridBagConstraints.CENTER);
		addComponent(this, op, 3, 1, 1, 1, GridBagConstraints.CENTER); 
		addComponent(this, valueT, 4, 1, 1, 1, GridBagConstraints.EAST); 
		addComponent(this, pc, 5, 1, 1, 1, GridBagConstraints.WEST); 
	}
	
	/**
	 * Get the type of the parameter search for this panel.
	 * @return true if its a tissue search or false in case of devStage search
	 */
	public boolean getTypeSearch() {
		return this.type;
	}
	
	/**
	 * Get the text of the filter box on this panel.
	 * @return a String containing the pattern
	 */		
	public String getEnteredPattern() {
		return this.patternT.getText();
	}
	
	/**
	 * Get the percent value entered by user.
	 * @return an integer which is the value entered
	 */		
	public int getValue() {
		return Integer.parseInt(this.valueT.getText());
	}
			
	/**
	 * Get the selected operation for this panel.
	 * @return a Character giving the operation selected
	 */
	public Character getOperation() {
		return (Character)op.getSelectedItem();
	}
	
	/**
	 * Get the unique panel number of this panel.
	 * @return the panel number
	 */
	public int getNumPanel() {
		return this.numPanel;	
	}	
	
	/**
	 * Get the 'Remove Criteria' button
	 * @return the remove JButton object 
	 */
	public JButton getRemoveButton() {
		return this.remove;
	}
	
	/**
	 * Get the 'Filter' button
	 * @return the filter JButton object 
	 */
	public JButton getFilterButton() {
		return this.filter;
	}
	
	/**
	 * Get the code corresponding to the selected item in the data ComboBox.
	 * @return the code corresponding to the selected item
	 */
	public int getDataBoxSelectedCode() {
		return Integer.parseInt((String)toShow[0][dataBox.getSelectedIndex()]);
	}
	
	/**
	 * Get the data parameter ComboBox.
	 * @return the data parameter ComboBox
	 */
	public JComboBox getDataBox() {
		return this.dataBox;
	}
	
	/**
	 * Set a new data parameter ComboBox.
	 * @param data the new data parameter ComboBox to set
	 */
	public void setDataBox(JComboBox data) {
		this.dataBox = data;
	}
	
	/**
	 * Update the item list show by the data parameter ComboBox.
	 * @param toShow an Object table which associate code with tissue or devStage
	 */
	public void setData(Object[][] toShow) {
		this.toShow = toShow;
	}
	
	/**
	 * Update the percent label by signaling an error with the entered value.
	 */
	public void setErrorPercentText() {
		this.pc.setText("% ERROR !");	
		this.pc.setIcon(new ImageIcon("./ico/ico-stop.gif"));			
	}
	
	/**
	 * Update the percent label by signaling a duplicate parameter.
	 */
	public void setErrorDuplicateText() {
		this.pc.setText("% DUPLICATE !");
		this.pc.setIcon(new ImageIcon("./ico/ico-stop.gif"));				
	}
	
	/**
	 * Update the percent label by signaling the 100% value reached.
	 */
	public void setErrorMaximumText() {
		this.pc.setText("% OVER MAX !");	
		this.pc.setIcon(new ImageIcon("./ico/ico-stop.gif"));			
	}
	
	/**
	 * Update the percent label by signaling no error with the entered value.
	 */
	public void setNoErrorPercentText() {
		this.pc.setText("% OK !");
		this.pc.setIcon(new ImageIcon("./ico/ico-check.gif"));		
	}
	
	/**
	 * Manage simply the layout to add component in the container. 
	 * Create a GridBagConstraints to add simply and more precisely components.
	 * @param container the container to update
	 * @param component the component to add
	 * @param x the horizontal position wanted
	 * @param y the vertical position wanted
	 * @param width the number of place to take in width
	 * @param height the number of place to take in height
	 * @param anchor the alignement position of the component
	 */ 
	private void addWithConstrains(Container container, Component component, 
		    			int x, int y, int width, int height, int anchor) {
	  Insets insets = new Insets (2, 2, 2, 2);
	  GridBagConstraints gbc = new GridBagConstraints();
	    	
	  gbc.gridx = x; gbc.gridy = y;
	  gbc.gridwidth = width; gbc.gridheight = height;
	  gbc.fill = GridBagConstraints.NONE; gbc.anchor = anchor;
	  gbc.weightx = 1.0; gbc.weighty = 1.0;
	  gbc.ipadx = 0; gbc.ipady = 0;
	  gbc.insets = insets;
	
	  LayoutManager lm = container.getLayout();
	  ((GridBagLayout) lm).setConstraints(component, gbc);
	  container.add(component);
	}
	
	/**
	 * Simply add a component to the container passed as argument.
	 * @param container the container to update
	 * @param component the component to add
	 * @param x the horizontal position wanted
	 * @param y the vertical position wanted
	 * @param width the number of place to take in width
	 * @param height the number of place to take in height
	 * @param position the alignement position of the component
	 */	  	
	protected void addComponent(Container container, Component component, 
		  											int x, int y, int width, int height, int position) {
		addWithConstrains(container, component, x, y, width, height, position);
	}
}
