package client;

import java.io.*;

/**
 * The class ClientSearchParameter represents a search parameter in 
 *  of the search panel.
 */
public class ClientSearchParameter implements Serializable {
	private int numPanel;
	private int wage;
	private boolean type; // false for a tissue, true for a devStage
	private int Hvalue;
	private Character op; // operations are   '>', '=', '<'
	private int value;

	/**
	 * Construct a new ClientSearchParameter object.
	 * @param numPanel the panel number of this parameter
	 * @param wage the wage associated with this parameter
	 * @param type the type of the parameter
	 * @param Hvalue the code of the parameter in the corresponding HashMap
	 * @param op the operation for this parameter
	 * @param value the percent value for this parameter
	 */
	public ClientSearchParameter(int numPanel, int wage, boolean type, int Hvalue, 
																										Character op, int value) {		
		this.numPanel = numPanel;
		this.wage = wage;
		this.type = type;
		this.Hvalue = Hvalue;	
		this.op = op;
		this.value = value;
	}
	
	/**
	 * Get the panel number associated with this parameter.
	 * @return the unique panel number
	 */
	public int getNumPanel() {
	  return this.numPanel;
	}
	
	/**
	 * Get the parameter wage.
	 * @return the wage of the parameter
	 */
	public int getWage() {
	  return this.wage;
	}
	
	/**
	 * Get the parameter's type.
	 * @return the type of the parameter
	 */
	public boolean getType() {
	  return this.type;
	}
	
	/**
	 * Set the parameter's type.
	 * @param type the type of the parameter
	 */
	public void setType(boolean type) {
	  this.type = type;
	}
	
	/**
	 * Get the parameter's hash key corresponding to the parameter.
	 * @return the parameter's hash key
	 */
	public int getHValue() {
	  return this.Hvalue;
	}
	
	/**
	 * Set a new parameter's hash key corresponding to the parameter.
	 * @param Hvalue the parameter's hash key to set
	 */
	public void setHValue(int Hvalue) {
	  this.Hvalue = Hvalue;
	}
	
	/**
	 * Get the operation selected for this parameter.
	 * @return a Character giving the parameter's operation
	 */
	public Character getOperation() {
	  return this.op;
	}
	
	/**
	 * Set the operation selected for this parameter.
	 * @param op the new operation to set
	 */
	public void setOperation(Character op) {
	  this.op = op;
	}
	
	/**
	 * Get the percent value of this parameter.
	 * @return the percent value
	 */
	public int getValue() {
	  return this.value;
	}
	
	/**
	 * Set a new percent value for this parameter.
	 * @param value the percent value to set
	 */
	public void setValue(int value) {
	  this.value = value;
	}
}