package client;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.awt.Color.*;

/**
 * This class represents the splash screen which is show
 *	while loading the bioECS client program.
 */
public class ClientSplash extends JDialog {	
	private JLabel splashImage = null;    
	private Dimension screensize = null;
	private JLabel statusString = null;
	private JProgressBar progressBar = null;
	
	/**
	 * Constructs a new ClientSplash screen object.
	 * @param image the picture of the window
	 * @param text the text of the label on top
	 */
	public ClientSplash(ImageIcon image, String text) {
		super();
		splashImage = new JLabel(image);
		setUndecorated(true);
		
		JPanel p = new JPanel(new GridLayout(2, 1));
		p.add(statusString = new JLabel());
		statusString.setBackground(Color.white);
		statusString.setBorder(
			BorderFactory.createLineBorder(new Color(75, 75, 75)));
		statusString.setForeground(Color.black);
		statusString.setOpaque(true);
		
		progressBar = new JProgressBar();
		progressBar.setMaximum(200);
		progressBar.setValue(0);
		progressBar.setBackground(Color.white);
		p.add(progressBar);
		
		this.getContentPane().add(new JLabel(text), BorderLayout.SOUTH);
		this.getContentPane().add(splashImage, BorderLayout.CENTER);
		this.getContentPane().add(p, BorderLayout.SOUTH);		
		this.pack();

		addWindowListener( new WindowAdapter() {
			public void windowDeactivated( WindowEvent e ) {
				toFront();
			}
		});

		splashImage.setBorder(
			BorderFactory.createLineBorder(new Color(75, 75, 75)));
	}

	/**
	 * Change the text.
	 * @param msg texte
	 */
	public void setText(String msg) {
		statusString.setText( " " + msg);
	}
					
	/**
	 * Change the value.
	 * @param val new value for status in %
	 */
	public void setValue(int val) {
		progressBar.setValue((progressBar.getMaximum()/100) * val);
	}
	
	/**
	 * Return the value.
	 * @return value of the progressbar 
	 */
	public int getValue() {
		return progressBar.getValue();
	}
	
	/**
	 * Change the text and the value.
	 * @param msg texte
	 * @param val new value for status in %
	 */
	public void setAll(String text, int val) {
		setText(text);
		setValue(val);
	}
	
	/**
	 * Set the frame in state visible or invisible, 
	 *	and set frame location on screen according to the screen size.
	 * @param b a boolean indicating the frame visibility value
	 */
	public void setVisible(boolean b){
		if (b) {
			Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		  int screenWidth  = screenSize.width;
		  int screenHeight = screenSize.height;
		  int frameWidth   = getSize().width;
		  int frameHeight  = getSize().height;
		  int posX         = (screenWidth - frameWidth) / 2;
		  int posY         = (screenHeight - frameHeight) / 2;
		  setLocation(posX, posY);
		}
		super.setVisible(b);
	}	    
}

