package client;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*; 
import javax.swing.table.*;
import javax.swing.border.TitledBorder;
import java.util.HashMap;
import java.util.ArrayList;


public class ClientTableCluster extends JPanel {	
	private ClientTableModelCluster modelClusterSearch;
	private JTable tableClusterSearch;
	private ClientTableRenderer label = null;		

	/**
	 * Create the cluster panel which list all the clusters found by a search.
	 */
	public ClientTableCluster(String title, ClientClusterSet ccs) {
		setBorder(BorderFactory.createTitledBorder(title));
		modelClusterSearch = new ClientTableModelCluster(ccs);	
	  
	  tableClusterSearch = new JTable(modelClusterSearch);	  
		tableClusterSearch.setRowSelectionAllowed(true); 
    tableClusterSearch.setSelectionMode(ListSelectionModel.SINGLE_SELECTION); 
    
		TableCellRenderer tbc = getTableCellRenderer();     
	  for (int i = 0; i < modelClusterSearch.getColumnCount(); ++i) {
	    TableColumn tc = tableClusterSearch.getColumnModel().getColumn(i);
	    tc.setCellRenderer(tbc);
	  }
		initClusterTableColumnSizes();
	  JScrollPane scrollPane = new JScrollPane(tableClusterSearch);

	  setLayout(new BorderLayout());
	  add(scrollPane, BorderLayout.CENTER);		 	  
	}
	
	/**
   * Initialize the columns' width of the Cluster Table.
   */
	private void initClusterTableColumnSizes() {
    ClientTableModelCluster model = 
    		(ClientTableModelCluster)tableClusterSearch.getModel();
    TableColumn column = null;
    Component comp = null;
    int headerWidth = 0;
    TableCellRenderer headerRenderer = 
    		tableClusterSearch.getTableHeader().getDefaultRenderer();

    for (int i = 0; i < model.getColumnCount(); ++i) {
      column = tableClusterSearch.getColumnModel().getColumn(i);
      comp = headerRenderer.getTableCellRendererComponent(
      			null, column.getHeaderValue(), false, false, 0, 0);
      if (i == 0) headerWidth = comp.getPreferredSize().width - 10;
      else headerWidth = comp.getPreferredSize().width;
      column.setPreferredWidth(headerWidth);
    }
  }
  
  /**
   * Add a table column header mouse listener.
   * @param ml the mouse listener to add
   */
  public void addHeaderListener(MouseListener ml) {
  	tableClusterSearch.getTableHeader().addMouseListener(ml);
  }
  
  /**
   * Add a table mouse listener.
   * @param ml the mouse listener to add
   */
  public void addListener(MouseListener ml) {
  	tableClusterSearch.addMouseListener(ml);
  }
   
  /**
   * Set the preferred scrollable viewport size of the table.
   * @param size the new dimension to set
   */
  public void setPanelSize(Dimension size) {
  	tableClusterSearch.setVisible(false);
  	tableClusterSearch.setPreferredScrollableViewportSize(size); 
    tableClusterSearch.setVisible(true);
  }
  
	/**
	 * Return the current cluster set.
	 * @return a ClientClusterSet object which is the current cluster set
	 */		 
	public ClientClusterSet getClusterSet() {
	  return this.modelClusterSearch.getClusterSet();
	}
	
	/**
	 * Set a new cluster set.
	 * @param cs the cluster set to set
	 */	
	public void setClusterSet(ClientClusterSet cs) {
	  this.modelClusterSearch.setClusterSet(cs);
	  this.modelClusterSearch.fireTableRowsInserted(0,0);
	}
	
	/**
	 * Set a cluster set in a savable state or not.
	 * @param savable the new savable state to set
	 */			
	public void setSavable(boolean savable) {
	  this.modelClusterSearch.setSavable(savable);
	}
	
	/**
	 * Get the savable state of the cluster set.
	 * @return savable the savable status
	 */	
	public boolean getSavable() {
	  return this.modelClusterSearch.getSavable();
	}
  
  /**
   * Get the table selected row.
   * @return an integer giving the selected row index
   */
  public int getSelectedRow() {
  	return tableClusterSearch.getSelectedRow();
  }
  
  /**
	 * Get the value to set at a (row,column) position in the model.
	 * @param row the row of the element in the model
	 * @param col the column of the element in the model
	 * @return an Object at position (row,col)
	 */
  public Object getValueAt(int row, int col) {
  	if ((row >= 0 && row < modelClusterSearch.getRowCount()) 
  			&& (col >= 0 && col < modelClusterSearch.getColumnCount()))
  		return modelClusterSearch.getValueAt(row, col);	
  	else return null;
	}
	
  /**
   * Return the ServerTableRenderer used by the program.
   * @return the TableCellRenderer object used
   */ 
  private TableCellRenderer getTableCellRenderer() {   	
    if (label == null)
      label = new ClientTableRenderer();
    return label;
  }
  
  /**
	 * Fusion sorting method which works recursively.
	 * The sorting task is done between indexes 'imin' & 'imax'.
	 * @param al the ArrayList to sort
	 * @param imin the minimum index
	 * @param imax the maximum index
	 * @param col the column to sort
	 */
	public void fusionSort(ArrayList al, int imin, int imax, int col) {
	  if (imin != imax) {
	    int mid = (imin + imax) / 2 + 1;
	    fusionSort(al, imin, mid - 1, col);
	    fusionSort(al, mid, imax, col);
	    fusion(al, imin, mid, imax, col);
	  }
	}

	/**
	 * Swap list's items to make the list sorted 
	 *	according to the 'double clicked' column number.
	 * The ArrayList al is sorted between 'imin' & 'mid-1' indexes, and
	 *	sorted between 'mid' & 'imax' indexes.
	 * Thus, the method sort al between 'imin' & 'imax' indexes.
	 *
	 * @param al the ArrayList to sort
	 * @param imin the minimum index
	 * @param mid is the centre index of the interval to sort
	 * @param imax the maximum index
	 * @param column the column to sort
	 */	 
	private void fusion(ArrayList al, int imin, int mid, int imax, int column) {
	  int i1=imin, i2=mid;
	  ArrayList tmpList = new ArrayList(imax - imin+1);
	  
	  for (int i=0; i < (imax - imin+1); i++) {
	    boolean leftChoose;
	    if (i2 > imax)
	       leftChoose = true;
	    else if (i1 >= mid)
	        leftChoose = false;
	    else {
	    	ClientCluster cc1 = (ClientCluster)al.get(i1);
	    	ClientCluster cc2 = (ClientCluster)al.get(i2);
	    	
	    	switch(column) {
					case 1: 
						double num = new Double(
							modelClusterSearch.getClusterSet()
							.getTotalNbESTMatching(cc1)).doubleValue();
						double den = new Double(cc1.getNbESTTotal()).doubleValue();
						Double result = new Double(num/den);							
						double numd = new Double(
							modelClusterSearch.getClusterSet()
							.getTotalNbESTMatching(cc2)).doubleValue();
						double dend = new Double(cc2.getNbESTTotal()).doubleValue();
						Double resultd = new Double(numd/dend);
												
						leftChoose = result.compareTo(resultd) >= 0;
						break;
					case 2:
						Integer cc1clu = new Integer(cc1.getClusterCode());
						Integer cc2clu = new Integer(cc2.getClusterCode());
						leftChoose = cc1clu.compareTo(cc2clu) <= 0;						
						break;
					case 3:
						Integer cc1gene = new Integer(cc1.getUnigeneCode());
						Integer cc2gene = new Integer(cc2.getUnigeneCode());
						leftChoose = cc1gene.compareTo(cc2gene) >= 0;						
						break;
					case 4:
						Integer cc1nb = new Integer(
							modelClusterSearch.getClusterSet().getTotalNbESTMatching(cc1));
						Integer cc2nb = new Integer(
							modelClusterSearch.getClusterSet().getTotalNbESTMatching(cc2));
						leftChoose = cc1nb.compareTo(cc2nb) >= 0;		
						break;	
					case 5:
						Integer cc1nbd = new Integer(
							modelClusterSearch.getClusterSet().getTotalNbESTMatching(cc1));
						Integer cc2nbd = new Integer(
							modelClusterSearch.getClusterSet().getTotalNbESTMatching(cc2));
						leftChoose = cc1nbd.compareTo(cc2nbd) >= 0;		
						break;	
					case 6:
						Integer cc1nbest = new Integer(cc1.getNbESTTotal());
						Integer cc2nbest = new Integer(cc2.getNbESTTotal());
						leftChoose = cc1nbest.compareTo(cc2nbest) >= 0;						
						break;	
					case 7:
						Integer cc1nbgo = new Integer(cc1.getNbGoFunctions());
						Integer cc2nbgo = new Integer(cc2.getNbGoFunctions());
						leftChoose = cc1nbgo.compareTo(cc2nbgo) >= 0;						
						break;
					case 8: 
						String desc1 = cc1.getDescription();
						String desc2 = cc2.getDescription();
						leftChoose = desc1.compareTo(desc2) <= 0;	
						break;
					default:
						leftChoose = false;	
						return;						
					}
	    }
	    if (leftChoose) {
	      tmpList.add(al.get(i1));
	      i1 = i1 + 1;
	    } 
	    else {
	      tmpList.add(al.get(i2));
	      i2 = i2 + 1;
	    }
	  }
	  for (int i = 0; i < tmpList.size(); i++)
	    al.set(imin + i, tmpList.get(i));
	}
  
  /**
   * The ClientTableRenderer's class defines the modified renderer used by
   *	all cells of the table specified.
   */ 
  private class ClientTableRenderer extends JLabel implements TableCellRenderer {    	    
    /**
     * Return the table renderer choosen for each cell
     * @param table the JTable object to consider
     * @param value the value to set
     * @param isSelected is true if the row is selected
     * @param hasFocus true if the component has currently the focus
     * @param row the row to consider
     * @param column the colmun to consider
     * @return a ServerTableRenderer object which is the renderer choosen
     */  
    public Component getTableCellRendererComponent(JTable table, Object value,
          				boolean isSelected, boolean hasFocus, int row, int column) {
      setOpaque(true);
      Font font = table.getFont();
      if (isSelected) {
        setBackground(new Color(180, 193, 210));
       	setForeground(Color.white); 
        font = new Font(font.getName(), Font.PLAIN | Font.BOLD, font.getSize());              
      }
      else {
        setBackground(Color.white); 
        setForeground(Color.black);          
        font = new Font(font.getName(), Font.PLAIN, font.getSize());
      }      
      setFont(font);
      setHorizontalAlignment(JLabel.CENTER);
      setText(value.toString());
      return this;
    }
  }
	
	/**
	 * The class ClientTableModelCluster is used to define 
	 *	the corresponding model scheme for a set of Clusters 
	 *	extracted from the database.
	 */
	private class ClientTableModelCluster extends AbstractTableModel {		
		private ClientClusterSet cs;
		private boolean savable;
		private ListenerModel listModel;
		private String[] columnNames = {
			"n",
			"% of EST matching",
			"TGI Code",
			"UniGene Code", 
			"Total nb of EST matching",
			"Matching details",
			"Total nb of EST",
			"nb of Go-Functions",
			"Description" 
		};
		
		/**
	 	 * Construct a new ClientTableModelCluster object.
		 * @param cs the cluster set to show
		 */		
		public ClientTableModelCluster(ClientClusterSet cs) {
			this.cs = cs;
		}
		
		/**
		 * Set a cluster set in a savable state or not.
		 * @param savable the new savable state to set
		 */			
		public void setSavable(boolean savable) {
		  this.savable = savable;
		}
		
		/**
		 * Get the savable state of the cluster set.
		 * @return savable the savable status
		 */	
		public boolean getSavable() {
		  return savable;
		}
		
		/**
	 	 * Return the current cluster set.
	 	 * @return a ClientClusterSet object which is the current cluster set
	 	 */		 
		public ClientClusterSet getClusterSet() {
		  return this.cs;
		}
		
		/**
	 	 * Set a new cluster set.
	 	 * @param cs the cluster set to set
	 	 */	
		public void setClusterSet(ClientClusterSet cs) {
		  this.cs = cs;
		}
		
		/**
		 * Create a listener for this model which is used to set savable 
		 *	the cluster set if it's updated
		 */
		private void addListenerModel() {
		  listModel = new ListenerModel();
		  this.addTableModelListener(listModel);
		}
		
		/**
		 * Get the value to set at a (row,column) position in the model.
		 * @param row the row of the element in the model
		 * @param col the column of the element in the model
		 * @return an object to show at position (row,col)
		 */
		public Object getValueAt(int row, int col) {	  		
		  switch(col) {
		   	case 0: 
		   		return new Integer(row+1);
		   	case 1: 	 		  			
		   		double num = new Double(
		   			cs.getTotalNbESTMatching(cs.getCluster(row))).doubleValue();		   		
					double den = new Double(
						cs.getCluster(row).getNbESTTotal()).doubleValue();
					String tmp = new Double((num/den)*100).toString();
					if (tmp.length() > 6)
						return new Double(tmp.substring(0, 7));
					return new Double(tmp);		
		    case 2: 
		    	String pref = "TC";
		    	String specie = cs.getSpecie();	 	
	  			if (specie.equals("Hs")) pref = "THC";
		    	return pref + cs.getCluster(row).getClusterCode();
		    case 3: 
		    	String spPref = cs.getSpecie();
		    	int unigeneCode = cs.getCluster(row).getUnigeneCode();
		    	if (unigeneCode > 0)	    		
		    		return new String(spPref + "." + unigeneCode);
		    	else return new String("---");
		    case 4: 
		    	return new Integer(cs.getTotalNbESTMatching(cs.getCluster(row)));
		    case 5: 
					return cs.getNbESTDetails(cs.getCluster(row));
		    case 6: 
		    	return new String(""+cs.getCluster(row).getNbESTTotal());
		    case 7: 
		    	return new Integer(cs.getCluster(row).getNbGoFunctions());
		    case 8: 
		    	return cs.getCluster(row).getDescription();
		    default: return null;
		  }
		}
		
		/**
		 * Get the number of columns of the model.
		 * @return an integer giving the number of columns
		 */
		public int getColumnCount() {
			return columnNames.length;
		}
		
		/**
		 * Get the number of rows of the model.
		 * @return an integer giving the number of rows
		 */
		public int getRowCount() {
	    return cs.getNbCluster();
		}
		
		/**
		 * Get the label name of a column.
		 * @param col specifying the column
		 * @return a String object containing the name of the column
		 */
		public String getColumnName(int col) {
		  return columnNames[col];
		}
		
		/**
		 * Get the class of the data type of a column.
		 * @param col specifying the column
		 * @return a Class object
		 */
		public Class getColumnClass(int col) {
		  return getValueAt(0, col).getClass();
		}
		
		/**
		 * Determine if a column is editable or not.
		 * @param row the row in the model
		 * @param col the row in the model
		 * @return true if the colmun is editable else return false
		 */
		public boolean isCellEditable(int row, int col) {
		  if (col == 8) return true;
		  else return false;
		}
		
		/**
		 * Set a value at a given position in the model.
		 * @param value the Object value to set
		 * @param row the row in the model
		 * @param col the row in the model
		 */
		public void setValueAt(Object value, int row, int col) {
		  if (col == 8) {
		    String s = (String)value;
		    cs.getCluster(row).setDescription(s);
		    fireTableDataChanged();
		  }	
		  fireTableCellUpdated(row, col);
		}
		
		/**
		 * This class is a listener used to automatically 
		 *	change the savable state of the model 
		 */
		public class ListenerModel implements TableModelListener {	
			public ListenerModel() {
		   	super();
		  }
		  
			public void tableChanged(TableModelEvent e) {
		  	if (e.getType() == TableModelEvent.UPDATE)
			  	savable = true;
		  }
		}
	}
}