package client;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*; 
import javax.swing.table.*;
import javax.swing.border.TitledBorder;
import java.util.HashMap;
import java.util.ArrayList;


public class ClientTableEST extends JPanel {	
	private ClientTableModelEST modelESTInfo;
	private JTable tableESTInfo;
	private ClientTableRenderer label = null;		
	
	HashMap HTitle = new HashMap();
	HashMap HTissue = new HashMap(); 
	HashMap HDevStage = new HashMap();
	
	/**
	 * Create the EST panel which list all the EST 
	 *	composing a cluster found by a search.
	 */
	public ClientTableEST(String title, ClientESTSet ces) {		
		setBorder(BorderFactory.createTitledBorder(title));
		modelESTInfo = new ClientTableModelEST(ces);

	  tableESTInfo = new JTable(modelESTInfo);
	  tableESTInfo.setRowSelectionAllowed(true); 
    tableESTInfo.setSelectionMode(ListSelectionModel.SINGLE_SELECTION); 

		TableCellRenderer tbc = getTableCellRenderer();     
	  for (int i = 0; i < modelESTInfo.getColumnCount(); ++i) {
	    TableColumn tc = tableESTInfo.getColumnModel().getColumn(i);
	    tc.setCellRenderer(tbc);
	  }
		initESTTableColumnSizes();			  
	  JScrollPane scrollPane = new JScrollPane(tableESTInfo);

	  setLayout(new BorderLayout());
	  add(scrollPane, BorderLayout.CENTER);		 	  		 
	}
	
	/**
   * Initialize the columns' width of the EST info Table.
   */
  private void initESTTableColumnSizes() {
    ClientTableModelEST model = 
    		(ClientTableModelEST)tableESTInfo.getModel();
    TableColumn column = null;
    Component comp = null;
    int headerWidth = 0;
    TableCellRenderer headerRenderer = 
    		tableESTInfo.getTableHeader().getDefaultRenderer();

    for (int i = 0; i < model.getColumnCount(); ++i) {
      column = tableESTInfo.getColumnModel().getColumn(i);
      comp = headerRenderer.getTableCellRendererComponent(
      			null, column.getHeaderValue(), false, false, 0, 0);
      if (i == 0) headerWidth = comp.getPreferredSize().width - 10;
      else headerWidth = comp.getPreferredSize().width;
      column.setPreferredWidth(headerWidth);
    }
  }
  
  /**
   * Add a table header mouse listener.
   * @param ml the mouse listener to add
   */
  public void addHeaderListener(MouseListener ml) {
  	tableESTInfo.getTableHeader().addMouseListener(ml);
  }
   
  /**
   * Set the preferred scrollable viewport size of the table.
   * @param size the new dimension to set
   */
  public void setPanelSize(Dimension size) {
  	tableESTInfo.setVisible(false);
  	tableESTInfo.setPreferredScrollableViewportSize(size); 
    tableESTInfo.setVisible(true);
  }
  
	/**
	 * Return the current EST Set.
	 * @return a ClientESTSet object which is the current EST Set
	 */	
	public ClientESTSet getESTSet() {
	  return this.modelESTInfo.getESTSet();
	}
	
	/**
	 * Set a new EST Set.
	 * @param ces the new EST Set to set
	 */
	public void setESTSet(ClientESTSet ces) {
	  this.modelESTInfo.setESTSet(ces);
	  this.modelESTInfo.fireTableRowsInserted(0,0);
	}
  
  /**
   * Get the table selected row.
   * @return an integer giving the selected row index
   */
  public int getSelectedRow() {
  	if (modelESTInfo.getRowCount() == 0 || modelESTInfo.getRowCount() == -1 ||
  				tableESTInfo.getSelectedRow() >= modelESTInfo.getRowCount())
  		return -1;
  	return tableESTInfo.getSelectedRow();
  }
  
  /**
	 * Get the value to set at a (row,column) position in the model.
	 * @param row the row of the element in the model
	 * @param col the column of the element in the model
	 * @return a String containing the value at position (row,col)
	 */
  public String getValueAt(int row, int col) {
  	if ((row >= 0 && row < modelESTInfo.getRowCount()) 
  			&& (col >= 0 && col < modelESTInfo.getColumnCount()))
  		return (String)modelESTInfo.getValueAt(row, col);	
  	else return "";
	}
	
  /**
   * Return the ServerTableRenderer used by the program.
   * @return the TableCellRenderer object used
   */ 
  private TableCellRenderer getTableCellRenderer() {   	
    if (label == null)
      label = new ClientTableRenderer();
    return label;
  }
  
  /**
	 * Fusion sorting method which works recursively.
	 * The sorting task is done between indexes 'imin' & 'imax'.
	 * @param al the ArrayList to sort
	 * @param imin the minimum index
	 * @param imax the maximum index
	 * @param col the column to sort
	 */
	public void fusionSort(ArrayList al, int imin, int imax, int col) {
	  if (imin != imax) {
	    int mid = (imin + imax) / 2 + 1;
	    fusionSort(al, imin, mid - 1, col);
	    fusionSort(al, mid, imax, col);
	    fusion(al, imin, mid, imax, col);
	  }
	}
		
	/**
	 * Swap list's items to make the list sorted 
	 *	according to the 'double clicked' column number.
	 * The ArrayList al is sorted between 'imin' & 'mid-1' indexes, and
	 *	sorted between 'mid' & 'imax' indexes.
	 * Thus, the method sort al between 'imin' & 'imax' indexes.
	 *
	 * @param al the ArrayList to sort
	 * @param imin the minimum index
	 * @param mid is the centre index of the interval to sort
	 * @param imax the maximum index
	 * @param column the column to sort
	 */
	private void fusion(ArrayList al, int imin, int mid, int imax, int column) {
	  int i1=imin, i2=mid;
	  ArrayList tmpList = new ArrayList(imax - imin+1);
	  
	  for (int i=0; i < (imax - imin+1); i++) {
	    boolean leftChoose;
	    if (i2 > imax)
	       leftChoose = true;
	    else if (i1 >= mid)
	        leftChoose = false;
	    else {
	    	ClientEST cc1 = (ClientEST)al.get(i1);
	    	ClientEST cc2 = (ClientEST)al.get(i2);
	    	
	    	switch(column) {
					case 1: 
						String est1 = cc1.getESTCode();
						String est2 = cc2.getESTCode();													
						leftChoose = est1.compareTo(est2) <= 0;
						break;
					case 2:
						String lib1 = (String)HTitle.get(""+cc1.getLibrary());
						String lib2 = (String)HTitle.get(""+cc2.getLibrary());
						leftChoose = lib1.compareTo(lib2) <= 0;						
						break;
					case 3:
						String tissue1 = (String)HTissue.get(""+cc1.getTissue());
						String tissue2 = (String)HTissue.get(""+cc2.getTissue());
						leftChoose = tissue1.compareTo(tissue2) <= 0;		
						break;	
					case 4:
						String ds1 = (String)HDevStage.get(""+cc1.getDevStage());
						String ds2 = (String)HDevStage.get(""+cc2.getDevStage());
						leftChoose = ds1.compareTo(ds2) <= 0;		
						break;			
					default:
						leftChoose = false;	
						return;					
					}
	    }
	    if (leftChoose) {
	      tmpList.add(al.get(i1));
	      i1 = i1 + 1;
	    } 
	    else {
	      tmpList.add(al.get(i2));
	      i2 = i2 + 1;
	    }
	  }
	  for (int i = 0; i < tmpList.size(); i++)
	    al.set(imin + i, tmpList.get(i));
	}
  
  /**
   * The ClientTableRenderer's class defines the modified renderer used by
   *	all cells of the table specified.
   */ 
  private class ClientTableRenderer extends JLabel implements TableCellRenderer {    	    
    /**
     * Return the table renderer choosen for each cell
     * @param table the JTable object to consider
     * @param value the value to set
     * @param isSelected is true if the row is selected
     * @param hasFocus true if the component has currently the focus
     * @param row the row to consider
     * @param column the colmun to consider
     * @return a ServerTableRenderer object which is the renderer choosen
     */  
    public Component getTableCellRendererComponent(JTable table, Object value,
          				boolean isSelected, boolean hasFocus, int row, int column) {
      setOpaque(true);
      Font font = table.getFont();
      if (isSelected) {
        setBackground(new Color(180, 193, 210));
       	setForeground(Color.white); 
        font = new Font(font.getName(), Font.PLAIN | Font.BOLD, font.getSize());              
      }
      else {
        setBackground(Color.white); 
        setForeground(Color.black);          
        font = new Font(font.getName(), Font.PLAIN, font.getSize());
      }      
      setFont(font);
      setHorizontalAlignment(JLabel.CENTER);
      setText(value.toString());
      return this;
    }
  }
	
	/**
	 * The class ClientTableModelEST is used to define 
	 *	the corresponding model scheme for a set of EST 
	 *	extracted from the database.
	 */
	private class ClientTableModelEST extends AbstractTableModel {		
		private ClientESTSet set;
		private String[] columnNames = {
			"n", 
			"EST NCBI Code", 
			"Library",
			"Tissue", 
			"Development Stage",
		};

		/**
	 	 * Construct a new ClientTableModelEST object.
		 * @param ces the EST Set to show
		 */	
		public ClientTableModelEST(ClientESTSet ces) {
			this.set = ces;
		}
	
		/**
	 	 * Return the current EST Set.
	 	 * @return a ClientESTSet object which is the current EST Set
	 	 */	
		public ClientESTSet getESTSet() {
		  return this.set;
		}
		
		/**
		 * Set a new EST Set.
		 * @param ces the new EST Set to set
		 */
		public void setESTSet(ClientESTSet ces) {
		  this.set = ces;
		}
	
		/**
		 * Get the value to set at a (row,column) position in the model.
		 * @param row the row of the element in the model
		 * @param col the column of the element in the model
		 * @return an object to show at position (row,col)
		 */
		public Object getValueAt(int row, int col) {
		  switch(col) {
		   	case 0: 
		   		return new Integer(row+1);
		    case 1: 
		    	return set.getEST(row).getESTCode();
		    case 2: 
		    	return (String)HTitle.get(
		    		new String(""+set.getEST(row).getLibrary()));
		    case 3: 
		    	return (String)HTissue.get(
		    		new String(""+set.getEST(row).getTissue()));
		    case 4: 
		    	return (String)HDevStage.get(
		    		new String(""+set.getEST(row).getDevStage()));
		    default: return null;
		  }
		}
		
		/**
		 * Get the number of columns of the model.
		 * @return an integer giving the number of columns
		 */
		public int getColumnCount() {
			return columnNames.length;
		}
		
		/**
		 * Get the number of rows of the model.
		 * @return an integer giving the number of rows
		 */
		public int getRowCount() {
			if (set == null)
				return -1;
		  return set.getNbEST();
		}
		
		/**
		 * Get the label name of a column.
		 * @param col specifying the column
		 * @return a String object containing the name of the column
		 */
		public String getColumnName(int col) {
		  return columnNames[col];
		}
		
		/**
		 * Get the class of the data type of a column.
		 * @param col specifying the column
		 * @return a Class object
		 */
		public Class getColumnClass(int col) {
		  return getValueAt(0, col).getClass();
		}
		
		/**
		 * Determine if a column is editable or not.
		 * @param row the row in the model
		 * @param col the row in the model
		 * @return always false for this model
		 */
		public boolean isCellEditable(int row, int col) {
			return false;
		}
		
		/**
		 * Set a value at a given position in the model.
		 * @param value the Object value to set
		 * @param row the row in the model
		 * @param col the row in the model
		 */
		public void setValueAt(Object value, int row, int col) {
		  fireTableCellUpdated(row, col);
		}
	}
}