package client;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*; 
import javax.swing.table.*;
import javax.swing.border.TitledBorder;
import java.util.HashMap;
import java.util.ArrayList;


public class ClientTableGO extends JPanel {	
	private ClientTableModelGO modelGOInfo;
	private JTable tableGOInfo;
	private ClientTableRenderer label = null;		
	
	/**
	 * Create the GO panel which list all the GO 
	 *	composing a cluster found by a search.
	 */
	public ClientTableGO(String title, ClientGOSet cgs) {		
		setBorder(BorderFactory.createTitledBorder(title));
		modelGOInfo = new ClientTableModelGO(cgs);

	  tableGOInfo = new JTable(modelGOInfo);
	  tableGOInfo.setRowSelectionAllowed(true); 
    tableGOInfo.setSelectionMode(ListSelectionModel.SINGLE_SELECTION); 

		TableCellRenderer tbc = getTableCellRenderer();     
	  for (int i = 0; i < modelGOInfo.getColumnCount(); ++i) {
	    TableColumn tc = tableGOInfo.getColumnModel().getColumn(i);
	    tc.setCellRenderer(tbc);
	  }
		initGOTableColumnSizes();			  
	  JScrollPane scrollPane = new JScrollPane(tableGOInfo);

	  setLayout(new BorderLayout());
	  add(scrollPane, BorderLayout.CENTER);		 	  		 
	}
	
	/**
   * Initialize the columns' width of the GO-Functions info Table.
   */
  private void initGOTableColumnSizes() {
    ClientTableModelGO model = (ClientTableModelGO)tableGOInfo.getModel();
    TableColumn column = null;
    Component comp = null;
    int headerWidth = 0;
    TableCellRenderer headerRenderer = 
    		tableGOInfo.getTableHeader().getDefaultRenderer();

    for (int i = 0; i < model.getColumnCount(); ++i) {
      column = tableGOInfo.getColumnModel().getColumn(i);
      comp = headerRenderer.getTableCellRendererComponent(
      			null, column.getHeaderValue(), false, false, 0, 0);
      if (i == 0) headerWidth = comp.getPreferredSize().width - 10;
      else headerWidth = comp.getPreferredSize().width;
      column.setPreferredWidth(headerWidth);
    }
  }

  /**
   * Add a table header mouse listener.
   * @param ml the mouse listener to add
   */
  public void addHeaderListener(MouseListener ml) {
  	tableGOInfo.getTableHeader().addMouseListener(ml);
  }
  
  /**
   * Set the preferred scrollable viewport size of the table.
   * @param size the new dimension to set
   */
  public void setPanelSize(Dimension size) {
  	tableGOInfo.setVisible(false);
  	tableGOInfo.setPreferredScrollableViewportSize(size); 
    tableGOInfo.setVisible(true);
  }

	/**
	 * Return the current GO Set.
	 * @return a ClientGOSet object which is the current GO Set
	 */	
	public ClientGOSet getGOSet() {
	  return this.modelGOInfo.getGOSet();
	}
	
	/**
	 * Set a new GO Set.
	 * @param cgos the new GO Set to set
	 */
	public void setGOSet(ClientGOSet cgos) {
	  this.modelGOInfo.setGOSet(cgos);
	  this.modelGOInfo.fireTableRowsInserted(0,0);
	}
  
  /**
   * Get the table selected row.
   * @return an integer giving the selected row index
   */
  public int getSelectedRow() {
  	if (modelGOInfo.getRowCount() == 0 || modelGOInfo.getRowCount() == -1 ||
  				tableGOInfo.getSelectedRow() >= modelGOInfo.getRowCount())
  		return -1;
  	return tableGOInfo.getSelectedRow();
  }
  
  /**
	 * Get the value to set at a (row,column) position in the model.
	 * @param row the row of the element in the model
	 * @param col the column of the element in the model
	 * @return a String containing the value at position (row,col)
	 */
  public String getValueAt(int row, int col) {
  	if ((row >= 0 && row < modelGOInfo.getRowCount()) 
  			&& (col >= 0 && col < modelGOInfo.getColumnCount()))
  		return (String)modelGOInfo.getValueAt(row, col);	
  	else return "";
	}
	
  /**
   * Return the ServerTableRenderer used by the program.
   * @return the TableCellRenderer object used
   */ 
  private TableCellRenderer getTableCellRenderer() {   	
    if (label == null)
      label = new ClientTableRenderer();
    return label;
  }
  
  /**
	 * Fusion sorting method which works recursively.
	 * The sorting task is done between indexes 'imin' & 'imax'.
	 * @param al the ArrayList to sort
	 * @param imin the minimum index
	 * @param imax the maximum index
	 * @param col the column to sort
	 */
	public void fusionSort(ArrayList al, int imin, int imax, int col) {
	  if (imin != imax) {
	    int mid = (imin + imax) / 2 + 1;
	    fusionSort(al, imin, mid - 1, col);
	    fusionSort(al, mid, imax, col);
	    fusion(al, imin, mid, imax, col);
	  }
	}
	
	/**
	 * Swap list's items to make the list sorted 
	 *	according to the 'double clicked' column number.
	 * The ArrayList al is sorted between 'imin' & 'mid-1' indexes, and
	 *	sorted between 'mid' & 'imax' indexes.
	 * Thus, the method sort al between 'imin' & 'imax' indexes.
	 *
	 * @param al the ArrayList to sort
	 * @param imin the minimum index
	 * @param mid is the centre index of the interval to sort
	 * @param imax the maximum index
	 * @param column the column to sort
	 */
	public void fusion(ArrayList al, int imin, int mid, int imax, int column) {
	  int i1=imin, i2=mid;
	  ArrayList tmpList = new ArrayList(imax - imin+1);
	  
	  for (int i=0; i < (imax - imin+1); i++) {
	    boolean leftChoose;
	    if (i2 > imax)
	       leftChoose = true;
	    else if (i1 >= mid)
	        leftChoose = false;
	    else {
	    	ClientGO cc1 = (ClientGO)al.get(i1);
	    	ClientGO cc2 = (ClientGO)al.get(i2);
	    	
	    	switch(column) {
					case 1: 
						Integer code1 = new Integer(cc1.getGOCode());
						Integer code2 = new Integer(cc2.getGOCode());												
						leftChoose = code1.compareTo(code2) <= 0;
						break;
					case 2:
						String term1 = cc1.getGOTerm();
						String term2 = cc2.getGOTerm();
						leftChoose = term1.compareTo(term2) <= 0;						
						break;
					case 3:
						String type1 = ""+cc1.getGOType();
						String type2 = ""+cc2.getGOType();
						leftChoose = type1.compareTo(type2) <= 0;		
						break;	
					case 4:
						String gopep1 = cc1.getGOpep();
						String gopep2 = cc2.getGOpep();
						leftChoose = gopep1.compareTo(gopep2) <= 0;		
						break;			
					default:
						leftChoose = false;	
						return;					
					}
	    }
	    if (leftChoose) {
	      tmpList.add(al.get(i1));
	      i1 = i1 + 1;
	    } 
	    else {
	      tmpList.add(al.get(i2));
	      i2 = i2 + 1;
	    }
	  }
	  for (int i = 0; i < tmpList.size(); i++)
	    al.set(imin + i, tmpList.get(i));
	}
  
  /**
   * The ClientTableRenderer's class defines the modified renderer used by
   *	all cells of the table specified.
   */ 
  private class ClientTableRenderer extends JLabel implements TableCellRenderer {    	    
    /**
     * Return the table renderer choosen for each cell
     * @param table the JTable object to consider
     * @param value the value to set
     * @param isSelected is true if the row is selected
     * @param hasFocus true if the component has currently the focus
     * @param row the row to consider
     * @param column the colmun to consider
     * @return a ServerTableRenderer object which is the renderer choosen
     */  
    public Component getTableCellRendererComponent(JTable table, Object value,
          				boolean isSelected, boolean hasFocus, int row, int column) {
      setOpaque(true);
      Font font = table.getFont();
      if (isSelected) {
        setBackground(new Color(180, 193, 210));
       	setForeground(Color.white); 
        font = new Font(font.getName(), Font.PLAIN | Font.BOLD, font.getSize());              
      }
      else {
        setBackground(Color.white); 
        setForeground(Color.black);          
        font = new Font(font.getName(), Font.PLAIN, font.getSize());
      }      
      setFont(font);
      setHorizontalAlignment(JLabel.CENTER);
      setText(value.toString());
      return this;
    }
  }
  
  /**
	 * The class ClientTableModelGO is used to define 
	 *	the corresponding table scheme.
	 */
	private class ClientTableModelGO extends AbstractTableModel {		
		private ClientGOSet set;
		private String[] columnNames = {
			"n", 
			"GO-Function ID", 
			"GO Term",
			"Type",
			"GOpep link",
		};
		
		/**
	 	 * Construct a new ClientTableModelGO object.
		 * @param cgos the GO Set to show
		 */	
		public ClientTableModelGO(ClientGOSet cgos) {
			this.set = cgos;
		}
	
		/**
	 	 * Return the current GO Set.
	 	 * @return a ClientGOSet object which is the current GO Set
	 	 */	
		public ClientGOSet getGOSet() {
		  return set;
		}
		
		/**
		 * Set a new GO Set.
		 * @param cgos the new GO Set to set
		 */
		public void setGOSet(ClientGOSet cgos) {
		  this.set = cgos;
		}
	
		/**
		 * Get the value to set at a (row,column) position in the model.
		 * @param row the row of the element in the model
		 * @param col the column of the element in the model
		 * @return an object to show at position (row,col)
		 */
		public Object getValueAt(int row, int col) {
		  switch(col) {
		   	case 0: 
		   		return new Integer(row+1);
		    case 1: 
		    	String toShow = "GO:";
		    	int code = set.getGOFunction(row).getGOCode();
		    	if (code < 10) toShow += "000000" + code;
		    	else if (code < 100) toShow += "00000" + code;
		    	else if (code < 1000) toShow += "0000" + code;
		    	else if (code < 10000) toShow += "000" + code;
		    	else if (code < 100000) toShow += "00" + code;
		    	else if (code < 1000000) toShow += "0" + code;	
		    	else toShow += code;
		    	return toShow;		    	
		    case 2: 
		    	return set.getGOFunction(row).getGOTerm();
		    case 3: 
		    	Character c = new Character(set.getGOFunction(row).getGOType());
		    	if (c.equals(new Character('P')))
		    		return new String("Biological Process - ("+c+")");
		    	else if (c.equals(new Character('C')))
		    		return new String("Cellular Component - ("+c+")");
		    	else if (c.equals(new Character('F')))
		    		return new String("Molecular Functions - ("+c+")");
		    case 4: 
		    	return set.getGOFunction(row).getGOpep();
		    default: return null;
		  }
		}
		
		/**
		 * Get the number of columns of the model.
		 * @return an integer giving the number of columns
		 */
		public int getColumnCount() {
			return columnNames.length;
		}
		
		/**
		 * Get the number of rows of the model.
		 * @return an integer giving the number of rows
		 */
		public int getRowCount() {
			if (set == null)
				return -1;
		  return set.getNbGOFunctions();
		}
		
		/**
		 * Get the label name of a column.
		 * @param col specifying the column
		 * @return a String object containing the name of the column
		 */
		public String getColumnName(int col) {
		  return columnNames[col];
		}
		
		/**
		 * Get the class of the data type of a column.
		 * @param col specifying the column
		 * @return a Class object
		 */
		public Class getColumnClass(int col) {
		  return getValueAt(0, col).getClass();
		}
		
		/**
		 * Determine if a column is editable or not.
		 * @param row the row in the model
		 * @param col the row in the model
		 * @return always false for this model
		 */
		public boolean isCellEditable(int row, int col) {
			return false;
		}
		
		/**
		 * Set a value at a given position in the model.
		 * @param value the Object value to set
		 * @param row the row in the model
		 * @param col the row in the model
		 */
		public void setValueAt(Object value, int row, int col) {
		  fireTableCellUpdated(row, col);
		}
	}
}