package client;

/**
 * The ClientThreadRetrieveEST is used to run the thread
 *	which will retrieve an ESTSet for a selected cluseter.
 */
public class ClientThreadRetrieveEST extends Thread {
	private String specieCode;
	private int clusterCode;
	private ClientRequestEST crm;
	private ClientESTSet ces;
	
	/**
	 * Construct a new ClientThreadRetrieveEST object.
	 * @param specieCode the specie selected
	 * @param clusterCode the selected cluster
	 * @param crm the request to execute
	 */	
	public ClientThreadRetrieveEST(String specieCode, int clusterCode, 
																											ClientRequestEST crm) {
		this.specieCode = specieCode;
		this.clusterCode = clusterCode;
		this.crm = crm;
		this.ces = new ClientESTSet(clusterCode);
	}	
	
	/**
	 * The run() method used to run the retrieval process.
	 */
	public void run() {		
		try {			
			this.ces = crm.checkESTInfo(specieCode, clusterCode);	
		}
		catch (Exception e) { 
			e.printStackTrace();
		} 
	}
	
	/**
	 * Get the ESTSet retrieved by the search.
	 * @return a ClientESTSet object which contains all the EST.
	 */
	public ClientESTSet getESTSet() {
		return this.ces;
	}
}