package client;

/**
 * The ClientThreadRetrieveGO is used to run the thread
 *	which will retrieve a GOSet for a selected cluseter.
 */
public class ClientThreadRetrieveGO extends Thread {
	private String specieCode;
	private int clusterCode;
	private ClientRequestGO crm;
	private ClientGOSet cgs;
	
	/**
	 * Construct a new ClientThreadRetrieveGO object.
	 * @param specieCode the specie selected
	 * @param clusterCode the selected cluster
	 * @param crm the request to execute
	 */		
	public ClientThreadRetrieveGO(String specieCode, int clusterCode, 
																										ClientRequestGO crm) {
		this.specieCode = specieCode;
		this.clusterCode = clusterCode;
		this.crm = crm;
		this.cgs = new ClientGOSet(clusterCode);
	}	
	
	/**
	 * The run() method used to run the retrieval process.
	 */
	public void run() {		
		try {
			this.cgs = crm.checkGOInfo(specieCode, clusterCode);	
		}
		catch (Exception e) { 
			e.printStackTrace();
		} 
	}
	
	/**
	 * Get the GOSet retrieved by the search.
	 * @return a ClientGOSet object which contains all the GO-Functions.
	 */
	public ClientGOSet getGOSet() {
		return this.cgs;
	}
}