package client;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import javax.swing.text.html.*; 
import java.awt.*;
import java.awt.event.*;
import java.net.URL;
import java.io.IOException;

/**
 * The ClientViewer class is used to retrieve 'online' 
 *	informations about a specified data.
 * It can retrieve more informations by connecting to the site given by url.
 */
public class ClientViewer extends JPanel {  

  private JEditorPane viewer;
  private JTextField urlField;
	private String url = "";
	private ClientParameters cp;
  
  /**
   * Construct a new ClientViewer object.
   * @param url the URL adress to connect
   * @param cp the client parameters
   */
  public ClientViewer(String url, ClientParameters cp) throws Exception {
  	boolean desactivated = false;
  	if (url.startsWith("#")) {
  		desactivated = true;
  		url = url.substring(1);
  	}		
    this.url = url;
    this.cp = cp;
    this.loadAutoResizer();
    
    JLabel comment = new JLabel(
    	"   Base url to copy and paste :   ", 
    	new ImageIcon("./ico/ico-network.png"), 0);     
    JButton remove = new JButton(
    	"", new ImageIcon("./ico/ico-network-delete.png"));
    remove.addActionListener(new ListRemove());       
    urlField = new JTextField(url);
    urlField.setEditable(false);  
         
    JPanel inputPanel = new JPanel(new BorderLayout());  
    inputPanel.add(comment, BorderLayout.WEST);
    inputPanel.add(urlField, BorderLayout.CENTER);
    inputPanel.add(remove, BorderLayout.EAST);

    viewer = new JEditorPane();
    viewer.setEditable(false);      
    JScrollPane scrollPane = new JScrollPane(viewer);
    
    if (!desactivated)   
    	loadPage(new URL(url));
     
		this.setLayout(new BorderLayout());
		this.add(inputPanel, BorderLayout.NORTH);
    this.add(scrollPane, BorderLayout.CENTER);
    this.validate();
		this.setVisible(true);
  }

  /**
	 * Load the frame's auto resizer module.
	 */
	private void loadAutoResizer() {
		this.addComponentListener(new ComponentAdapter() {
    	public void componentResized(ComponentEvent evt) { 
      	repaint();
    	}
  	});
	}

  
  /**
   * Load a WWW page given by the URL.
   * @param url the URL to connect
   */     
  private void loadPage(URL url) {
  	try {		      	
			viewer.setPage(url);
		}
		catch(IOException ioe) {
			PrintConnectionError(url.toString(), ioe); 
		}
  }

	/**
	 * Listener used to remove this tab.
	 */
  public class ListRemove implements ActionListener, Runnable {
    public void actionPerformed(ActionEvent event) {     	
			new Thread(this).start();
    }
    
    public void run() {
    	ClientViewer.this.getParent().remove(ClientViewer.this);
    	viewer = null;
    	System.gc();
    }
  }

  /**
   * Print a connection error message if the debug mode is selected.
   * @param ex the exception to print
   */
  private void PrintConnectionError(String url, Exception ex) {
		String msg = "Can't connect to " + url;
		msg += "\n  Please check your Internet access configuration"; 
		if (cp.PRINT_DEBUG_INFO) {
			System.out.println("** Viewer Exception :");
	    System.out.println("*  Message: " + ex.getMessage());
	  	System.out.println("*  Can't connect to "+url);
	  }
	  connectionFailedFrame(ClientViewer.this, msg);
	}
	
	/**
	 * Show a 'connection failed frame' displaying error possible sources.
	 * @param panel the parent panel associated.
	 * @param msg the message to show.
	 */
  private void connectionFailedFrame(JPanel panel, String msg) {
    JOptionPane.showMessageDialog(panel, msg, 
    	"Connection failed !", JOptionPane.WARNING_MESSAGE, 
    	new ImageIcon("./ico/bioecs.png"));
         
  }
  
  /**
	 * Show a 'Desactivated' link frame.
	 * @param panel the parent panel associated.
	 */
  private void connectionDesactivatedFrame(JPanel panel) {
  	String msg = "Sorry, this link is not accessible in bioECS !";
    JOptionPane.showMessageDialog(panel, msg, 
    	"Desactivated link !", JOptionPane.WARNING_MESSAGE, 
    	new ImageIcon("./ico/bioecs.png"));     
  }
}
